from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0002_auto_20160406_1548"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="photo",
            options={"ordering": ("-post_time",)},
        ),
        migrations.AlterField(
            model_name="photo",
            name="fetch_time",
            field=models.DateTimeField(
                blank=True,
                help_text="The time the item's data was last fetched.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="fetch_time",
            field=models.DateTimeField(
                blank=True, help_text="The time the data was last fetched.", null=True
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="iconserver",
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# Generated by Django 5.1.5 on 2025-02-05 13:23

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('lastfm', '0008_auto_20180104_1457'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scrobble',
            name='post_time',
            field=models.DateTimeField(blank=True, db_index=True, default=django.utils.timezone.now, help_text='The time the item was originally posted/created on its service.', null=True),
        ),
    ]



from django.apps import AppConfig


class DittoTwitterConfig(AppConfig):
    name = "ditto.twitter"
    verbose_name = "Ditto Twitter"

    # Maintain pre Django 3.2 default behaviour:
    default_auto_field = "django.db.models.AutoField"



# Generated by Django 1.9.5 on 2016-07-13 14:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0049_auto_20160713_1127"),
    ]

    operations = [
        migrations.AlterField(
            model_name="media",
            name="media_type",
            field=models.CharField(
                choices=[
                    ("animated_gif", "Animated GIF"),
                    ("photo", "Photo"),
                    ("video", "Video"),
                ],
                max_length=12,
            ),
        ),
    ]



# Generated by Django 1.9.5 on 2016-06-15 10:45

from django.db import migrations, models

import ditto.twitter.models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0047_media_video_file"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="media",
            name="video_file",
        ),
        migrations.AddField(
            model_name="media",
            name="mp4_file",
            field=models.FileField(
                blank=True,
                default="",
                help_text="Only used for Animated GIFs",
                upload_to=ditto.twitter.models.Media.upload_path,
            ),
        ),
    ]



# Generated by Django 1.9.5 on 2016-06-03 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0041_remove_media_tweet"),
    ]

    operations = [
        migrations.AlterField(
            model_name="media",
            name="tweets",
            field=models.ManyToManyField(related_name="media", to="twitter.Tweet"),
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-19 12:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0055_re_save_tweets_for_new_html"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="tweet",
            options={"ordering": ["-post_time", "-twitter_id"]},
        ),
        migrations.AddIndex(
            model_name="tweet",
            index=models.Index(
                fields=["user", "post_time", "is_private"],
                name="twitter_twe_user_id_12e113_idx",
            ),
        ),
    ]



# Generated by Django 1.9.5 on 2016-06-03 13:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0038_tweet_retweeted_status_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="media",
            name="tweets",
            field=models.ManyToManyField(to="twitter.Tweet"),
        ),
        migrations.AlterField(
            model_name="media",
            name="tweet",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="temporary_name",
                to="twitter.Tweet",
            ),
        ),
    ]



from django.core.management.base import CommandError

from ditto.core.management.commands import DittoBaseCommand


class FetchCommand(DittoBaseCommand):
    """
    Parent for all classes that fetch some things from Flickr. Photos,
    Photosets, Files, etc.
    """

    def add_arguments(self, parser):
        "All children will have the --account option."
        super().add_arguments(parser)

        parser.add_argument(
            "--account",
            action="store",
            default=False,
            help=(
                "The NSID of the Flickr User associated with the one "
                "Account to fetch for."
            ),
        )


class FetchPhotosCommand(FetchCommand):
    # What we're fetching:
    singular_noun = "Photo"
    plural_noun = "Photos"

    # Child classes should supply some help text for the --days and
    # --start/--end arguments:
    days_help = ""
    start_help = ""
    end_help = ""

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument("--days", action="store", default=None, help=self.days_help)

        parser.add_argument(
            "--start", action="store", default=None, help=self.start_help
        )
        parser.add_argument("--end", action="store", default=None, help=self.end_help)

    def handle(self, *args, **options):
        # We might be fetching for a specific account or all (None).
        nsid = options["account"] if options["account"] else None

        if options["days"] and (options["start"] or options["end"]):
            msg = "You can't use --days with --start or --end"
            raise CommandError(msg)

        if options["days"]:
            # Will be either 'all' or a number; make the number an int.
            if options["days"].isdigit():
                options["days"] = int(options["days"])
            elif options["days"] != "all":
                msg = "--days should be an integer or 'all'."
                raise CommandError(msg)

            results = self.fetch_photos(nsid=nsid, days=options["days"])
            self.output_results(results, options.get("verbosity", 1))

        elif options["start"] or options["end"]:
            results = self.fetch_photos(
                nsid=nsid, start=options["start"], end=options["end"]
            )
            self.output_results(results, options.get("verbosity", 1))

        elif options["account"]:
            msg = "Specify --days, or --start and/or --end as well as --account."
            raise CommandError(msg)

        else:
            msg = "Specify --days, or --start and/or --end."
            raise CommandError(msg)

    def fetch_photos(self, nsid, days=None, start=None, end=None):
        """Child classes should override this method to call a method that
        fetches photos and returns results, eg:
            return RecentPhotosMultiAccountFetcher(nsid=nsid).fetch(days=days)
        """
        return {}



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0004_auto_20160414_1120"),
    ]

    operations = [
        migrations.RenameField(
            model_name="photo",
            old_name="height_mp4_hd",
            new_name="hd_mp4_height",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="width_mp4_hd",
            new_name="hd_mp4_width",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="height_mp4_mobile",
            new_name="mobile_mp4_height",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="width_mp4_mobile",
            new_name="mobile_mp4_width",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="height_mp4_site",
            new_name="site_mp4_height",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="width_mp4_site",
            new_name="site_mp4_width",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_b",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_c",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_h",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_k",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_m",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_n",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_o",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_t",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_video_original",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="height_z",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_b",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_c",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_h",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_k",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_m",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_n",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_o",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_t",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_video_original",
        ),
        migrations.RemoveField(
            model_name="photo",
            name="width_z",
        ),
        migrations.AddField(
            model_name="photo",
            name="large_1600_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_1600_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_2400_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_2400_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_640_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_640_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_800_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_800_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="original_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="original_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_320_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_320_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="thumbnail_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="thumbnail_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="video_original_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="video_original_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0010_auto_20150730_1112"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="api_token",
            field=models.CharField(
                help_text='From https://pinboard.in/settings/password eg, "philgyford:1234567890ABCDEFGHIJ"',  # noqa: E501
                max_length=51,
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="time_created",
            field=models.DateTimeField(
                help_text="The time this item was created in the database.",
                auto_now_add=True,
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="time_modified",
            field=models.DateTimeField(
                help_text="The time this item was last saved to the database.",
                auto_now=True,
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="url",
            field=models.URLField(
                unique=True,
                help_text="eg, 'https://pinboard.in/u:philgyford'",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="username",
            field=models.CharField(
                unique=True, help_text="eg, 'philgyford'", max_length=30
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="description",
            field=models.TextField(
                help_text="The 'extended' text description.", blank=True
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="fetch_time",
            field=models.DateTimeField(
                help_text="The time the item's data was last fetched, and was new or changed.",  # noqa: E501
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="is_private",
            field=models.BooleanField(
                help_text="If set, this item will not be shown on public-facing pages.",
                default=False,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="permalink",
            field=models.URLField(
                help_text="URL of the item on the service's website.", blank=True
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="post_time",
            field=models.DateTimeField(
                help_text="The time this was created on Pinboard.",
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="raw",
            field=models.TextField(
                help_text="eg, the raw JSON from the API.", blank=True
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="summary",
            field=models.CharField(
                help_text="eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.",  # noqa: E501
                blank=True,
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="time_created",
            field=models.DateTimeField(
                help_text="The time this item was created in the database.",
                auto_now_add=True,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="time_modified",
            field=models.DateTimeField(
                help_text="The time this item was last saved to the database.",
                auto_now=True,
            ),
        ),
    ]



# Generated by Django 1.10.1 on 2016-11-21 13:24

from django.db import migrations, models


def set_post_year(apps, schema_editor):
    """
    Sets the `post_year` value on every Photo.
    """
    Photo = apps.get_model("flickr", "Photo")
    for row in Photo.objects.all():
        row.post_year = row.post_time.year
        row.save()


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0020_auto_20161117_1622"),
    ]

    operations = [
        migrations.AddField(
            model_name="photo",
            name="post_year",
            field=models.PositiveSmallIntegerField(
                blank=True,
                db_index=True,
                help_text="Set automatically on save",
                null=True,
            ),
        ),
        migrations.RunPython(set_post_year, reverse_code=migrations.RunPython.noop),
    ]



import json
import os
from datetime import datetime, timezone

from django.conf import settings
from django.core.files import File

from ditto.core.utils import truncate_string
from ditto.core.utils.downloader import DownloadException, filedownloader
from ditto.twitter.models import Media, Tweet, User

# Classes that take JSON data from the Twitter API and create or update
# objects.
#
# They don't call the API at all. But UserSaver fetches avatar images.

# CLASSES HERE:
#
# SaveUtilsMixin
# UserSaver
# TweetSaver


class SaveUtilsMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _api_time_to_datetime(self, api_time, time_format="%a %b %d %H:%M:%S +0000 %Y"):
        """Change a text datetime from the API to a datetime with timezone.
        api_time is a string like 'Wed Nov 15 16:55:59 +0000 2006'.
        """
        return datetime.strptime(api_time, time_format).replace(tzinfo=timezone.utc)


class UserSaver(SaveUtilsMixin):
    "Provides a method for creating/updating a User using data from the API."

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save_user(self, user, fetch_time, *, download_avatar=True):
        """With Twitter user data from the API, it creates or updates the User
        and returns the User object.

        Keyword arguments:
        user -- A dict of the data about a user from the API's JSON.
        fetch_time -- A datetime.
        download_avatar -- Boolean. Should user's profile pic be downloaded?

        Returns the User object.
        """
        raw_json = json.dumps(user)

        defaults = {
            "fetch_time": fetch_time,
            "raw": raw_json,
            "screen_name": user["screen_name"],
            "name": user["name"],
            "is_verified": user["verified"],
            "profile_image_url_https": user["profile_image_url_https"],
        }

        # When ingesting tweets there are lots of fields the 'user' element
        # doesn't have, compared to the API:

        if "protected" in user:
            defaults["is_private"] = user["protected"]

        if "created_at" in user:
            defaults["created_at"] = self._api_time_to_datetime(user["created_at"])

        # If there's a URL it will be a t.co shortened one.
        # So we go through the entities to find its expanded version.
        if "url" in user and user["url"]:
            user_url = user["url"]
            if "url" in user["entities"] and "urls" in user["entities"]["url"]:
                for url_dict in user["entities"]["url"]["urls"]:
                    if (
                        url_dict["url"] == user["url"]
                        and url_dict["expanded_url"] is not None
                    ):
                        user_url = url_dict["expanded_url"]
            defaults["url"] = user_url

        if "description" in user:
            defaults["description"] = user["description"] if user["description"] else ""

        if "location" in user:
            defaults["location"] = user["location"] if user["location"] else ""

        if "time_zone" in user:
            defaults["time_zone"] = user["time_zone"] if user["time_zone"] else ""

        if "favourites_count" in user:
            defaults["favourites_count"] = user["favourites_count"]

        for a_count in [
            "followers_count",
            "friends_count",
            "listed_count",
            "statuses_count",
        ]:
            if a_count in user:
                defaults[a_count] = user[a_count]

        user_obj, created = User.objects.update_or_create(
            twitter_id=user["id"], defaults=defaults
        )

        if download_avatar:
            user_obj = self._fetch_and_save_avatar(user_obj)

        return user_obj

    def _fetch_and_save_avatar(self, user):
        """
        Download and save the Avatar/profile pic for this user.
        If the user's profile_image_url_https property doesn't match an image
        we've already downloaded for them, we fetch and save it.

        user -- User object.
        """
        filename = os.path.basename(user.profile_image_url_https)
        # Where we'd save the user's avatar image to:
        avatar_upload_path = os.path.join(
            settings.MEDIA_ROOT, user.avatar_upload_path(filename)
        )
        if os.path.exists(avatar_upload_path):
            return user

        # We don't have this image yet, so fetch and save it.
        try:
            avatar_filepath = filedownloader.download(
                user.profile_image_url_https,
                ["image/jpeg", "image/jpg", "image/png", "image/gif"],
            )

            with open(avatar_filepath, "rb") as f:
                user.avatar.save(os.path.basename(avatar_filepath), File(f))
        except DownloadException:
            pass

        return user


class TweetSaver(SaveUtilsMixin):
    """Provides a method for creating/updating a Tweet (and its User) using
    data from the API. Also used by ingest.TweetIngester()
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save_media(self, tweet):
        """Takes a Tweet object and creates or updates any photos and videos
        based on the JSON data in its `raw` field.

        Keyword arguments:
        tweet -- The Tweet object. Must have been saved as we need its id.

        Returns:
        Total number of items for this Tweet (regardless of whether they were
            created or updated).
        """
        import itertools

        # What we'll return:
        media_count = 0

        try:
            json_data = json.loads(tweet.raw)
        except ValueError:
            return media_count

        try:
            media = json_data["extended_entities"]["media"]
        except KeyError:
            return media_count

        for item in media:
            # Things common to photos, animated GIFs and videos.

            defaults = {
                "media_type": "photo",
                "twitter_id": item["id"],
                "image_url": item["media_url_https"],
            }

            valid_types = [type for type, name in Media.MEDIA_TYPES]

            if item["type"] in valid_types:
                defaults["media_type"] = item["type"]

            for size in ["large", "medium", "small", "thumb"]:
                if size in item["sizes"]:
                    defaults[size + "_w"] = item["sizes"][size]["w"]
                    defaults[size + "_h"] = item["sizes"][size]["h"]
                else:
                    defaults[size + "_w"] = None
                    defaults[size + "_h"] = None

            # Adding things ony used for videos and animated GIFs:
            if "video_info" in item:
                info = item["video_info"]

                # eg, '16:9'
                defaults["aspect_ratio"] = "{}:{}".format(
                    info["aspect_ratio"][0],
                    info["aspect_ratio"][1],
                )

                if "duration_millis" in info:
                    defaults["duration"] = info["duration_millis"]

                # Group the variants into a dict with keys like 'video/mp4'
                # and values being a list of dicts. Each of those dicts
                # being a single variant.
                # [
                #   {'video/mp4': [{'url':'...', 'bitrate':320000},
                #                  {'url':'...', 'bitrate':832000},]
                #   },
                # ]
                # A bit less necessary now that mp4s are deprecated for videos.
                # (But there's still a single(?) mp4 url for animated GIFs.)

                # Sort the list of dicts by 'content_type'.
                # Need to do this for the next grouping stage.
                sorted_variants = sorted(
                    info["variants"], key=lambda k: k["content_type"]
                )
                grouped_variants = {}

                for key, variants in itertools.groupby(
                    sorted_variants, lambda k: k["content_type"]
                ):
                    grouped_variants[key] = list(variants)

                if "application/dash+xml" in grouped_variants:
                    defaults["dash_url"] = grouped_variants["application/dash+xml"][0][
                        "url"
                    ]
                if "application/x-mpegURL" in grouped_variants:
                    defaults["xmpeg_url"] = grouped_variants["application/x-mpegURL"][
                        0
                    ]["url"]

                if (
                    defaults["media_type"] == "animated_gif"
                    and "video/mp4" in grouped_variants
                ):
                    # Only Animated GIFs have mp4s now.
                    # https://twittercommunity.com/t/retiring-mp4-video-output/66093
                    defaults["mp4_url"] = grouped_variants["video/mp4"][0]["url"]

            media_obj, created = Media.objects.update_or_create(
                twitter_id=item["id"], defaults=defaults
            )
            media_count += 1

            if tweet not in media_obj.tweets.all():
                media_obj.tweets.add(tweet)

        return media_count

    def save_tweet(self, tweet, fetch_time, user_data=None):
        """Takes a dict of tweet data from the API and creates or updates a
        Tweet object and its associated User object.

        Keyword arguments:
        tweet -- The tweet data.
        fetch_time -- A datetime.

        Returns:
        The Tweet object that was created or updated.
        """
        raw_json = json.dumps(tweet)
        try:
            created_at = self._api_time_to_datetime(tweet["created_at"])
        except ValueError:
            # Because the tweets imported from a downloaded archive have a
            # different format for created_at. Of course. Why not?!
            created_at = self._api_time_to_datetime(
                tweet["created_at"], time_format="%Y-%m-%d %H:%M:%S +0000"
            )

        if user_data is None:
            if "user" in tweet:
                user_data = tweet["user"]
            else:
                msg = "No user data found to save tweets with"
                raise ValueError(msg)

        user = UserSaver().save_user(user_data, fetch_time)

        if "full_text" in tweet:
            # For new (2016) 'extended' format tweet data.
            # https://dev.twitter.com/overview/api/upcoming-changes-to-tweets
            text = tweet["full_text"]
            # Cuts off any @usernames at the start and a trailing URL at the end:
            frm = int(tweet["display_text_range"][0])
            to = int(tweet["display_text_range"][1])
            title = text[frm:to]
        else:
            # Older 'classic' format tweet data.
            text = tweet["text"]
            title = text

        # titles can only be 255 characters
        title = truncate_string(
            title, strip_html=True, chars=255, truncate="…", at_word_boundary=True
        )

        defaults = {
            "fetch_time": fetch_time,
            "raw": raw_json,
            "user": user,
            "is_private": user.is_private,
            "post_time": created_at,
            "permalink": "https://twitter.com/{}/status/{}".format(
                user.screen_name, tweet["id"]
            ),
            "title": title.replace("\n", " ").replace("\r", " "),
            "text": text,
            "twitter_id": tweet["id"],
            "source": tweet["source"],
        }

        # Some of these are only present in tweets from the API (not in the
        # tweets from the downloaded archive).
        # Some are just not always present, such as coordinates and place stuff.

        if "favorite_count" in tweet:
            defaults["favorite_count"] = tweet["favorite_count"]

        if "retweet_count" in tweet:
            defaults["retweet_count"] = tweet["retweet_count"]

        if "lang" in tweet:
            defaults["language"] = tweet["lang"]

        if (  # noqa: SIM102
            "coordinates" in tweet
            and tweet["coordinates"]
            and "type" in tweet["coordinates"]
        ):
            if tweet["coordinates"]["type"] == "Point":
                defaults["latitude"] = tweet["coordinates"]["coordinates"][1]
                defaults["longitude"] = tweet["coordinates"]["coordinates"][0]
            # TODO: Handle Polygons?

        if "in_reply_to_screen_name" in tweet and tweet["in_reply_to_screen_name"]:
            defaults["in_reply_to_screen_name"] = tweet["in_reply_to_screen_name"]
        else:
            defaults["in_reply_to_screen_name"] = ""

        if "in_reply_to_status_id" in tweet:
            defaults["in_reply_to_status_id"] = tweet["in_reply_to_status_id"]

        if "in_reply_to_user_id" in tweet:
            defaults["in_reply_to_user_id"] = tweet["in_reply_to_user_id"]

        if "place" in tweet and tweet["place"] is not None:
            if (
                "attributes" in tweet["place"]
                and "street_address" in tweet["place"]["attributes"]
            ):
                defaults["place_attribute_street_address"] = tweet["place"][
                    "attributes"
                ]["street_address"]
            if "full_name" in tweet["place"]:
                defaults["place_full_name"] = tweet["place"]["full_name"]
            if "country" in tweet["place"]:
                defaults["place_country"] = tweet["place"]["country"]

        if "quoted_status_id" in tweet:
            defaults["quoted_status_id"] = tweet["quoted_status_id"]

            if "quoted_status" in tweet:
                # If tweet 1 quotes tweet 2 that quotes tweet 3, then
                # tweet 2 will have 'quoted_status_id' but not 'quoted_status'.
                # But the tweet does have quoted_status, we'll create/update
                # the quoted User object, and quoted Tweet.
                UserSaver().save_user(tweet["quoted_status"]["user"], fetch_time)
                self.save_tweet(tweet["quoted_status"], fetch_time)

        if "retweeted_status" in tweet:
            defaults["retweeted_status_id"] = tweet["retweeted_status"]["id"]

            UserSaver().save_user(tweet["retweeted_status"]["user"], fetch_time)
            self.save_tweet(tweet["retweeted_status"], fetch_time)

        tweet_obj, created = Tweet.objects.update_or_create(
            twitter_id=tweet["id"], defaults=defaults
        )

        # Create/update any Photos, and update the Tweet's photo_count:
        media_count = self.save_media(tweet=tweet_obj)
        tweet_obj.media_count = media_count

        tweet_obj.save()

        return tweet_obj



import calendar
import datetime
import os
import time

import flickrapi
from django.core.files import File
from flickrapi.exceptions import FlickrError

from ditto.core.utils import datetime_now
from ditto.core.utils.downloader import DownloadException, filedownloader
from ditto.flickr.models import Account, User

from . import FetchError
from .savers import PhotoSaver, PhotosetSaver, UserSaver

# These classes call the Flickr API to fetch data about particular things,
# from the point of view of a single Account. eg, Photos, Users, Photosets.
#
# This data is then passed on to *Saver classes for creation/updating of
# objects.

# CLASSES HERE:
#
# Fetcher
#   UserIdFetcher
#   UserFetcher
#   PhotosFetcher
#       RecentPhotosFetcher
#   PhotosetsFetcher


class Fetcher:
    """Parent class for children that will call the Flickr API to fetch data.

    Depending on the child classes, it would be used something like:

        results = Fetcher(account=account_object).fetch()

    results is a dict that will have:
        'success': Boolean.
        'account': String. Indicating the Account (eg, its User's username).
        'fetched': Integer. If success, the number of things fetched, if any.
        'messages': List. If no success, the failure message(s).
    """

    # How many photos (or whatever) do we fetch per page of retults?
    items_per_page = 500

    def __init__(self, account):
        # Will be an Account object, passed into init()
        self.account = None

        # Will be the FlickrAPI object for calling the Flickr API.
        self.api = None

        # Will be incremented if we're fetching multiple pages.
        self.page_number = 1

        # If there are multiple pages to fetch, this should be set when we get
        # the first page.
        self.total_pages = 1

        # Will be the results fetched from the API via FlickrAPI.
        self.results = []

        # When fetching Photos or Users this will be the total number fetched.
        self.results_count = 0

        # What we'll return:
        self.return_value = {"fetched": 0}

        if isinstance(account, Account):
            if account.user:
                self.return_value["account"] = account.user.username
            elif account.pk:
                self.return_value["account"] = f"Account: {account}"
            else:
                self.return_value["account"] = "Unsaved Account"
        else:
            msg = "An Account object is required"
            raise ValueError(msg)

        if account.has_credentials():
            self.account = account
            self.api = flickrapi.FlickrAPI(
                self.account.api_key, self.account.api_secret, format="parsed-json"
            )
        else:
            self.return_value["success"] = False
            self.return_value["messages"] = ["Account has no API credentials"]

    def fetch(self, **kwargs):
        self._fetch_pages(**kwargs)

        if self._not_failed():
            self.return_value["success"] = True
            self.return_value["fetched"] = self.results_count

        return self.return_value

    def _fetch_pages(self, **kwargs):
        """Fetch all of the pages.
        This relies on something else setting self.total_pages to the correct
        value. Probably in _call_api(), set it when fetching the first page,
        as the results will contain the total number of pages available.
        """
        while self.page_number <= self.total_pages and self._not_failed():
            self._fetch_page(**kwargs)
            self.page_number += 1
            time.sleep(0.5)  # Being nice.

    def _fetch_page(self, **kwargs):
        try:
            self._call_api(**kwargs)
        except FetchError as e:
            self.return_value["success"] = False
            self.return_value["messages"] = [f"Error when calling Flickr API: {e}"]
            return

        try:
            self._fetch_extra()
        except FetchError as e:
            self.return_value["success"] = False
            self.return_value["messages"] = [f"Error when fetching extra data: {e}"]
            return

        try:
            self._save_results()
            # Clear for the next page:
            self.results = []
        except FetchError as e:
            self.return_value["success"] = False
            self.return_value["messages"] = [f"Error when saving data: {e}"]
            return

        return

    def _not_failed(self):
        """Has everything gone smoothly so far? ie, no failure registered?"""
        return (
            "success" not in self.return_value or self.return_value["success"] is True
        )

    def _call_api(self, **kwargs):
        """
        Should call self.api.a_function() and set self.results with the results.
        """
        msg = "Subclasess of Fetcher should define their own _call_api()."
        raise FetchError(msg)

    def _fetch_extra(self):
        """Can be defined in subclasses to fetch extra data to add to
        self.results before we save the data in the DB."""

    def _save_results(self, **kwargs):
        """
        Should go through self.results and create/update things in the DB based
        on its contents.
        """
        msg = "Subclasses of Fetcher should define their own _save_results()."
        raise FetchError(msg)


class UserIdFetcher(Fetcher):
    """Returns the Flickr ID for an Account's user.
    Doesn't save anything.

    Use like:
        results = UserIdFetcher(account=account_obj).fetch()

    Results will look something like:
        {
            'id': '35034346050@N01',
            'fetched': 1,
            'account': 'Phil Gyford',
            'success': True,
        }
    """

    def _call_api(self):
        """Calls test.login() to get the very basic user info for the
        authenticating user.
        Docs: https://www.flickr.com/services/api/flickr.test.login.htm
        """
        try:
            info = self.api.test.login()
        except FlickrError as err:
            msg = f"Error when calling test.login(): {err}'"
            raise FetchError(msg) from err

        self.results = [{"id": info["user"]["id"]}]

    def _save_results(self):
        "Doesn't save any results, just prepares to return the Flickr ID."
        self.return_value["id"] = self.results[0]["id"]
        self.results_count = 1


class UserFetcher(Fetcher):
    """Fetches and saves data about a single User by Flickr ID."""

    def fetch(self, nsid=None):
        """Fetches and saves data about a single User by Flickr ID.
        nsid -- A Flickr ID for a user.
        """
        if nsid is None:
            msg = "UserFetcher().fetch() requires a Flickr id (NSID)"
            raise FetchError(msg)

        return super().fetch(nsid=nsid)

    def _call_api(self, nsid):
        "nsid -- A Flickr ID for a user."
        try:
            info = self.api.people.getInfo(user_id=nsid)
        except FlickrError as err:
            # User has deleted their account, so create a dummy result
            if err.code == 5:
                self.results = [
                    {
                        "raw": "{}",
                        "nsid": nsid,
                        "username": {"_content": "deleted_user_" + nsid},
                        "photos": {
                            "count": {"_content": "0"},
                            "firstdate": {"_content": "0"},
                            "firstdatetaken": {"_content": "1970-01-01 00:00:00"},
                        },
                    }
                ]
                return

            msg = f"Error when getting info about User with Flickr ID '{nsid}': {err}"
            raise FetchError(msg) from err

        # info has 'person' and 'stat' elements.
        self.results = [info["person"]]

    def _save_results(self):
        user_obj = UserSaver().save_user(self.results[0], datetime_now())
        self._fetch_and_save_avatar(user_obj)
        self.return_value["user"] = {"name": user_obj.name}
        self.results_count = 1

    def _fetch_and_save_avatar(self, user):
        """
        Download and save the Avatar/profile pic for this user.
        user -- User object.
        """
        try:
            avatar_filepath = filedownloader.download(
                user.original_icon_url,
                ["image/jpeg", "image/jpg", "image/png", "image/gif"],
            )

            with open(avatar_filepath, "rb") as f:
                user.avatar.save(os.path.basename(avatar_filepath), File(f))
        except DownloadException:
            pass


class PhotosFetcher(Fetcher):
    """Parent class for fetching and saving data about Photos for an Account."""

    def __init__(self, *args, **kwargs):
        # Will match Flickr IDs with their User object.
        # eg '35034346050@N01' => User
        # When we fetch a user's data, because we need it to save a photo/tag,
        # we add their object to this, so we don't fetch again this time.
        self.fetched_users = {}

        super().__init__(*args, **kwargs)

    def _call_api(self):
        """
        Should call self.api.a_function() and set self.results with the results.
        """
        msg = "Subclasess of PhotosFetcher should define their own _call_api()."
        raise FetchError(msg)

    def _fetch_extra(self):
        """Before saving we need to go through the big list of photos we've
        fetched, and fetch more detailed info to add to each photo's data.
        """
        extra_results = []

        for _i, photo in enumerate(self.results):
            self._fetch_user_if_missing(photo["owner"])

            extra_results.append(
                {
                    "fetch_time": datetime_now(),
                    # Add the data for the photo's owner:
                    "user_obj": self.fetched_users[photo["owner"]],
                    # Get all the info about this photo:
                    "info": self._fetch_photo_info(photo["id"]),
                    "sizes": self._fetch_photo_sizes(photo["id"]),
                    "exif": self._fetch_photo_exif(photo["id"]),
                }
            )

        # Replace self.results with our new array that contains more info.
        self.results = extra_results

    def _fetch_user_if_missing(self, flickr_user_id):
        """
        If we don't have flickr_user_id in self.fetched_users, then fetch, and
        save, that user from the API. Then add their User to self.fetched_users.
        flickr_user_id -- The user's ID on Flickr, eg '35034346050@N01'.
        """
        if self.fetched_users.get(flickr_user_id, None) is None:
            results = UserFetcher(account=self.account).fetch(nsid=flickr_user_id)
            if results["success"] is False:
                raise FetchError(results["messages"][0])

            # Get the user we just saved. A bit clunky!
            self.fetched_users[flickr_user_id] = User.objects.get(nsid=flickr_user_id)

    def _fetch_photo_info(self, photo_id):
        """Calls the photos.getInfo() method of the Flickr API and returns the
        info about the photo.
        https://www.flickr.com/services/api/explore/flickr.photos.getInfo
        photo_id -- The Flickr photo ID.
        """
        try:
            results = self.api.photos.getInfo(photo_id=photo_id)
        except FlickrError as err:
            msg = f"Error when fetching photo info (photo {photo_id}): {err}"
            raise FetchError(msg) from err

        # Each tag on the photo is added by a specific Flickr user.
        # (Usually, but not always, the photo owner.)
        # Check that we've got info about that user in our DB.
        for tag in results["photo"]["tags"]["tag"]:
            self._fetch_user_if_missing(tag["author"])

        return results["photo"]

    def _fetch_photo_sizes(self, photo_id):
        """Calls the photos.getSizes() method of the Flickr API and returns the
        photo's sizes.
        https://www.flickr.com/services/api/explore/flickr.photos.getSizes
        photo_id -- The Flickr photo ID.
        """
        try:
            results = self.api.photos.getSizes(photo_id=photo_id)
        except FlickrError as err:
            msg = f"Error when fetching photo sizes (photo {photo_id}): {err}"
            raise FetchError(msg) from err
        return results["sizes"]

    def _fetch_photo_exif(self, photo_id):
        """Calls the photos.getExif() method of the Flickr API and returns the
        photo's EXIF data.
        https://www.flickr.com/services/api/explore/flickr.photos.getExif
        photo_id -- The Flickr photo ID.
        """
        try:
            results = self.api.photos.getExif(photo_id=photo_id)
        except FlickrError as err:
            msg = f"Error when fetching photo EXIF data (photo {photo_id}): {err}"
            raise FetchError(msg) from err
        return results["photo"]

    def _save_results(self):
        """Save all the data we've fetched about photos to the DB."""
        saver = PhotoSaver()
        for photo in self.results:
            saver.save_photo(photo)
        self.results_count += len(self.results)


class RecentPhotosFetcher(PhotosFetcher):
    """Fetches and saves data about an Account's recent Photos.

    By default ALL photos will be fetched.

    Supply a number of days to fetch() to restrict to the most recent days.
    """

    def __init__(self, account):
        super().__init__(account)

        # Maximum date of photos to return, if days or start are passed in:
        # By default, set it before Flickr so we get everything.
        self.min_date = datetime.datetime.strptime("2000-01-01", "%Y-%m-%d").astimezone(
            datetime.timezone.utc
        )

        # Maximum date of photos to return, if end is passed in:
        self.max_date = None

    def fetch(self, days=None, start=None, end=None):
        """Fetch all of the Account's user's photos, by default.
        days  - The number of days back to look, by upload date, or
                'all' to fetch all photos.
        start - The start date of a range in YYYY-MM-DD,YYYY-MM-DD format
        end   - The end date of a range in YYYY-MM-DD,YYYY-MM-DD format
        """

        if days and (start or end):
            msg = "You can't use --days with --start or --end"
            raise ValueError(msg)

        if days:
            try:
                self.min_date = datetime_now() - datetime.timedelta(days=days)
            except TypeError as err:
                if days != "all":
                    msg = "days should be an integer or 'all'."
                    raise FetchError(msg) from err
        elif start or end:
            try:
                if start:
                    self.min_date = datetime.datetime.strptime(
                        f"{start} 00:00:00", "%Y-%m-%d %H:%M:%S"
                    ).astimezone(datetime.timezone.utc)

                if end:
                    self.max_date = datetime.datetime.strptime(
                        f"{end} 23:59:59", "%Y-%m-%d %H:%M:%S"
                    ).astimezone(datetime.timezone.utc)

                if (start and end) and (start > end):
                    msg = "Start date must be before the end date."
                    raise ValueError(msg)

            except TypeError as err:
                msg = (
                    "Something went wrong with start or end. Please check the date "
                    "format. It should be YYYY-MM-DD"
                )
                raise FetchError(msg) from err
        else:
            msg = "Either set days or start and/or end."
            raise FetchError(msg)

        return super().fetch()

    def _call_api(self):
        """Fetch one page of results, containing very basic info about the
        Photos."""

        # Set up default arguments dictionary
        api_args = {
            "user_id": self.account.user.nsid,
            "per_page": self.items_per_page,
            "page": self.page_number,
        }

        if self.min_date:
            # Turn our datetime object into a unix timestamp and add to arguments:
            min_unixtime = calendar.timegm(self.min_date.timetuple())
            api_args["min_upload_date"] = min_unixtime

        if self.max_date:
            # Turn our datetime object into a unix timestamp and add to arguments:
            max_unixtime = calendar.timegm(self.max_date.timetuple())
            api_args["max_upload_date"] = max_unixtime

        try:
            results = self.api.people.getPhotos(**api_args)
        except FlickrError as err:
            msg = f"Error when fetching recent photos (page {self.page_number}: {err}"
            raise FetchError(msg) from err

        if (
            self.page_number == 1
            and "photos" in results
            and "pages" in results["photos"]
        ):
            # First time, set the total_pages there are to fetch.
            self.total_pages = int(results["photos"]["pages"])

        # Add the list of photos' data from this page on to our total list:
        self.results += results["photos"]["photo"]


class PhotosetsFetcher(Fetcher):
    def _call_api(self):
        """Fetch one page of results.

        https://www.flickr.com/services/api/flickr.photosets.getList.htm
        """

        try:
            results = self.api.photosets.getList(
                user_id=self.account.user.nsid,
                per_page=self.items_per_page,
                page=self.page_number,
            )
        except FlickrError as err:
            msg = f"Error when fetching photosets (page {self.page_number}): {err}"
            raise FetchError(msg) from err

        if (
            self.page_number == 1
            and "photosets" in results
            and "pages" in results["photosets"]
        ):
            # First time, set the total_pages there are to fetch.
            self.total_pages = int(results["photosets"]["pages"])

        # Add the list of photosets' data from this page on to our total list:
        self.results += results["photosets"]["photoset"]

    def _fetch_extra(self):
        """Before saving we need to get the list of photos in each photoset."""

        extra_results = []

        for _i, photoset in enumerate(self.results):
            photos = self._fetch_photos_in_photoset(photoset["id"])

            extra_results.append(
                {
                    "fetch_time": datetime_now(),
                    "photoset": photoset,
                    "photos": photos,
                    "user_obj": self.account.user,
                }
            )

        # Replace self.results with our new array that contains more info.
        self.results = extra_results

    def _fetch_photos_in_photoset(self, photoset_id):
        """Gets the info about all the photos in the photoset.
        Might be in multiple pages.
        The data is quite minimal, but that's all we need.

        https://www.flickr.com/services/api/flickr.photosets.getPhotos.htm

        Expects:
            photoset_id -- eg 72157665648859705
        Returns:
            list of dicts, each dict containing photo data
        """

        photos = []
        page_number = 1
        total_pages = 1  # Will get set to its proper value below.

        while page_number <= total_pages:
            try:
                results = self.api.photosets.getPhotos(
                    photoset_id=photoset_id,
                    user_id=self.account.user.nsid,
                    per_page=self.items_per_page,
                    page=page_number,
                )
            except FlickrError as err:
                msg = (
                    f"Error when fetching photos in photoset {photoset_id} "
                    f"(page {page_number}): {err}"
                )
                raise FetchError(msg) from err

            if "photoset" in results and "photo" in results["photoset"]:
                total_pages = results["photoset"]["pages"]
                photos += results["photoset"]["photo"]

            page_number += 1
            time.sleep(0.5)  # Being nice.

        return photos

    def _save_results(self):
        """Save all the data we've fetched about photosets to the DB."""
        saver = PhotosetSaver()
        for photoset in self.results:
            saver.save_photoset(photoset)
        self.results_count += len(self.results)



# Generated by Django 1.9.5 on 2016-04-28 15:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0019_auto_20160428_1525"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bookmark",
            name="url_hash",
            field=models.CharField(
                db_index=True,
                help_text="Slug in the Bookmark's URL here.",
                max_length=12,
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0014_auto_20151028_1556"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bookmark",
            name="post_time",
            field=models.DateTimeField(
                help_text="The time the item was originally posted/created on its service.",  # noqa: E501
                blank=True,
                null=True,
            ),
        ),
    ]



from imagekit import ImageSpec, register
from imagekit.processors import Adjust, ResizeToFill, ResizeToFit, Transpose

# Info about different Flickr image sizes:
# https://www.flickr.com/services/api/misc.urls.html
#
# Also see ditto.flickr.models.Photo


# BASE CLASSES


class FlickrSpec(ImageSpec):
    "Base class for most specs. Keeps same proportions."

    format = "JPEG"
    options = {"quality": 80}
    # If original image is smaller, don't enlarge it:
    upscale = False

    def __init__(self, source):
        self.processors = [
            Transpose(),
            ResizeToFit(self.width, self.height, upscale=self.upscale),
            Adjust(sharpness=2.0),
        ]
        super().__init__(source)


class FlickrSquareSpec(ImageSpec):
    "Base class for the square specs. Crops to a square."

    format = "JPEG"
    options = {"quality": 70}
    # If original image is smaller, enlarge it to fill:
    upscale = True

    def __init__(self, source):
        self.processors = [
            Transpose(),
            ResizeToFill(self.size, self.size, upscale=self.upscale),
            Adjust(sharpness=2.0),
        ]
        super().__init__(source)


# SQUARES


class Square(FlickrSquareSpec):
    size = 75


register.generator("ditto_flickr:square", Square)


class LargeSquare(FlickrSquareSpec):
    size = 150


register.generator("ditto_flickr:large_square", LargeSquare)


# NATURAL PROPORTIONS


class Thumbnail(FlickrSpec):
    width = 100
    height = 100
    # If original image is smaller, enlarge it to fit:
    upscale = True


register.generator("ditto_flickr:thumbnail", Thumbnail)


class Small(FlickrSpec):
    width = 240
    height = 240


register.generator("ditto_flickr:small", Small)


class Small320(FlickrSpec):
    width = 320
    height = 320


register.generator("ditto_flickr:small_320", Small320)


class Medium(FlickrSpec):
    width = 500
    height = 500


register.generator("ditto_flickr:medium", Medium)


class Medium640(FlickrSpec):
    width = 640
    height = 640


register.generator("ditto_flickr:medium_640", Medium640)


class Medium800(FlickrSpec):
    width = 800
    height = 800


register.generator("ditto_flickr:medium_800", Medium800)


class Large(FlickrSpec):
    width = 1024
    height = 1024


register.generator("ditto_flickr:large", Large)


class Large1600(FlickrSpec):
    width = 1600
    height = 1600


register.generator("ditto_flickr:large_1600", Large1600)


class Large2048(FlickrSpec):
    width = 2048
    height = 2048


register.generator("ditto_flickr:large_2048", Large2048)


class XLarge3K(FlickrSpec):
    width = 3072
    height = 3072


register.generator("ditto_flickr:x_large_3k", XLarge3K)


class XLarge4K(FlickrSpec):
    width = 4096
    height = 4096


register.generator("ditto_flickr:x_large_4k", XLarge4K)


class XLarge5K(FlickrSpec):
    width = 5120
    height = 5120


register.generator("ditto_flickr:x_large_5k", XLarge5K)


class XLarge6K(FlickrSpec):
    width = 6000
    height = 6000


register.generator("ditto_flickr:x_large_6k", XLarge6K)



import factory

from ditto.core.utils import datetime_now

from . import models


class AccountFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Account

    username = factory.Sequence(lambda n: f"user{n:04}")
    realname = factory.Sequence(lambda n: f"Real Name {n:04}")
    api_key = factory.Sequence(lambda n: f"api-key-{n:04}")


class ArtistFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Artist

    name = factory.Sequence(lambda n: f"Artist {n:04}")
    slug = factory.Sequence(lambda n: f"artist+{n:04}")
    original_slug = factory.Sequence(lambda n: f"Artist+{n:04}")


class TrackFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Track

    name = factory.Sequence(lambda n: f"Track {n:04}")
    slug = factory.Sequence(lambda n: f"track+{n:04}")
    original_slug = factory.Sequence(lambda n: f"Track+{n:04}")
    artist = factory.SubFactory(ArtistFactory)


class AlbumFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Album

    name = factory.Sequence(lambda n: f"Album {n:04}")
    slug = factory.Sequence(lambda n: f"album+{n:04}")
    original_slug = factory.Sequence(lambda n: f"Album+{n:04}")
    artist = factory.SubFactory(ArtistFactory)


class ScrobbleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Scrobble

    account = factory.SubFactory(AccountFactory)
    track = factory.SubFactory(TrackFactory)
    artist = factory.SubFactory(ArtistFactory)
    post_time = datetime_now()



# Generated by Django 1.9.5 on 2016-05-16 10:36

from django.db import migrations, models

import ditto.flickr.models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0015_auto_20160514_1456"),
    ]

    operations = [
        migrations.AddField(
            model_name="photo",
            name="video_original_file",
            field=models.FileField(
                blank=True,
                default="",
                help_text="Only present for Videos.",
                upload_to=ditto.flickr.models.Photo.upload_path,
            ),
        ),
    ]



from django.contrib import admin
from django.db import models
from django.forms import TextInput
from django.utils.safestring import mark_safe

from ditto.core.admin import DittoItemModelAdmin

from .models import Account, Photo, Photoset, User


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = (
        "user",
        "has_credentials",
        "is_active",
        "time_created",
        "time_modified",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "user",
                    "is_active",
                )
            },
        ),
        (
            "API",
            {
                "fields": (
                    "api_key",
                    "api_secret",
                ),
                "description": (
                    "Keys and secrets require creation of an app at "
                    '<a href="https://www.flickr.com/services/apps/create/apply/">'
                    "flickr.com/...</a>"
                ),
            },
        ),
        (
            "Data",
            {
                "classes": ("collapse",),
                "fields": (
                    "time_created",
                    "time_modified",
                ),
            },
        ),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {"widget": TextInput(attrs={"class": "vLargeTextField"})},
    }

    readonly_fields = (
        "time_created",
        "time_modified",
    )

    def has_credentials(self, obj):
        return obj.has_credentials()

    has_credentials.boolean = True


@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    list_display = (
        "show_avatar",
        "realname",
        "username",
        "fetch_time",
    )
    list_display_links = (
        "show_avatar",
        "realname",
        "username",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "realname",
                    "username",
                    "nsid",
                    "is_pro",
                    "location",
                    "description",
                    "photos_url",
                    "profile_url",
                    "avatar",
                    "photos_count",
                    "photos_views",
                    "photos_first_date",
                    "photos_first_date_taken",
                    "timezone_id",
                )
            },
        ),
        (
            "Data",
            {
                "classes": ("collapse",),
                "fields": (
                    "iconserver",
                    "iconfarm",
                    "raw",
                    "fetch_time",
                    "time_created",
                    "time_modified",
                ),
            },
        ),
    )
    readonly_fields = (
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
    )
    search_fields = (
        "realname",
        "username",
    )

    def show_avatar(self, instance):
        return mark_safe(f'<img src="{instance.avatar_url}" width="24" height="24" />')

    show_avatar.short_description = ""


@admin.register(Photoset)
class PhotosetAdmin(admin.ModelAdmin):
    list_display = (
        "title",
        "flickr_created_time",
        "fetch_time",
    )
    list_display_links = ("title",)

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "user",
                    "title",
                    "description",
                    "primary_photo",
                    "flickr_id",
                    "photo_count",
                    "video_count",
                    "view_count",
                    "comment_count",
                    "last_update_time",
                    "flickr_created_time",
                    "photos",
                )
            },
        ),
        (
            "Data",
            {
                "classes": ("collapse",),
                "fields": (
                    "raw",
                    "fetch_time",
                    "time_created",
                    "time_modified",
                ),
            },
        ),
    )
    readonly_fields = (
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
    )
    search_fields = (
        "title",
        "description",
    )
    raw_id_fields = (
        "primary_photo",
        "photos",
    )


class TaggedPhotoInline(admin.TabularInline):
    model = Photo.tags.through
    raw_id_fields = ("tag",)


@admin.register(Photo)
class PhotoAdmin(DittoItemModelAdmin):
    list_display = (
        "title",
        "show_thumb",
        "post_time",
        "taken_time",
    )
    list_display_links = (
        "title",
        "show_thumb",
    )
    list_filter = (
        "post_time",
        "fetch_time",
        "media",
        "safety_level",
        "license",
    )
    date_hierarchy = "post_time"
    search_fields = [
        "title",
        "description",
    ]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "show_image",
                    "user",
                    "title",
                    "description",
                    "summary",
                    "permalink",
                    "is_private",
                    "flickr_id",
                    "media",
                    "license",
                    "original_file",
                    "video_original_file",
                )
            },
        ),
        (
            "Times",
            {
                "classes": ("collapse",),
                "fields": (
                    "post_time",
                    "post_year_str",
                    "last_update_time",
                    "taken_time",
                    "taken_year_str",
                    "taken_granularity",
                    "taken_unknown",
                ),
            },
        ),
        (
            "Counts, secrets, server, etc",
            {
                "classes": ("collapse",),
                "fields": (
                    "view_count",
                    "comment_count",
                    "secret",
                    "original_secret",
                    "server",
                    "farm",
                    "rotation",
                    "original_format",
                    "safety_level",
                    "has_people",
                ),
            },
        ),
        (
            "Sizes",
            {
                "classes": ("collapse",),
                "fields": (
                    (
                        "thumbnail_width",
                        "thumbnail_height",
                    ),
                    (
                        "small_width",
                        "small_height",
                    ),
                    (
                        "small_320_width",
                        "small_320_height",
                    ),
                    (
                        "medium_width",
                        "medium_height",
                    ),
                    (
                        "medium_640_width",
                        "medium_640_height",
                    ),
                    (
                        "medium_800_width",
                        "medium_800_height",
                    ),
                    (
                        "large_width",
                        "large_height",
                    ),
                    (
                        "large_1600_width",
                        "large_1600_height",
                    ),
                    (
                        "large_2048_width",
                        "large_2048_height",
                    ),
                    (
                        "x_large_3k_width",
                        "x_large_3k_height",
                    ),
                    (
                        "x_large_4k_width",
                        "x_large_4k_height",
                    ),
                    (
                        "x_large_5k_width",
                        "x_large_5k_height",
                    ),
                    (
                        "x_large_6k_width",
                        "x_large_6k_height",
                    ),
                    (
                        "original_width",
                        "original_height",
                    ),
                    (
                        "mobile_mp4_width",
                        "mobile_mp4_height",
                    ),
                    (
                        "site_mp4_width",
                        "site_mp4_height",
                    ),
                    (
                        "hd_mp4_width",
                        "hd_mp4_height",
                    ),
                    (
                        "video_original_width",
                        "video_original_height",
                    ),
                ),
            },
        ),
        (
            "Location",
            {
                "classes": ("collapse",),
                "fields": (
                    "geo_is_private",
                    (
                        "latitude",
                        "longitude",
                    ),
                    "location_accuracy",
                    "location_context",
                    "location_place_id",
                    "location_woeid",
                    "locality_name",
                    "locality_place_id",
                    "locality_woeid",
                    "county_name",
                    "county_place_id",
                    "county_woeid",
                    "region_name",
                    "region_place_id",
                    "region_woeid",
                    "country_name",
                    "country_place_id",
                    "country_woeid",
                ),
            },
        ),
        (
            "EXIF",
            {
                "classes": ("collapse",),
                "fields": (
                    "exif_camera",
                    "exif_lens_model",
                    "exif_aperture",
                    "exif_exposure",
                    "exif_flash",
                    "exif_focal_length",
                    "exif_iso",
                ),
            },
        ),
        (
            "Data",
            {
                "classes": ("collapse",),
                "fields": (
                    "raw",
                    "sizes_raw",
                    "exif_raw",
                    "fetch_time",
                    "time_created",
                    "time_modified",
                ),
            },
        ),
    )
    radio_fields = {"media": admin.HORIZONTAL}
    readonly_fields = (
        "post_year_str",
        "taken_year_str",
        "show_image",
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
        "sizes_raw",
        "exif_raw",
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {"widget": TextInput(attrs={"class": "vLargeTextField"})},
    }
    inlines = [
        TaggedPhotoInline,
    ]
    raw_id_fields = ("user",)

    def show_thumb(self, instance):
        return mark_safe(
            f'<img src="{instance.thumbnail_url}" '
            f'width="{instance.thumbnail_width}" '
            f'height="{instance.thumbnail_height}" />'
        )

    show_thumb.short_description = "Thumbnail"

    def show_image(self, instance):
        return mark_safe(
            f'<img src="{instance.small_url}" '
            f'width="{instance.small_width}" '
            f'height="{instance.small_height}" />'
        )

    show_image.short_description = "Small image"

    def taken_year_str(self, instance):
        "So Admin doesn't add a comma, like '2,016'."
        return str(instance.taken_year)

    taken_year_str.short_description = "Taken year"



# Generated by Django 4.0.2 on 2022-02-14 10:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0056_add_count_index"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="access_token",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Access Token"
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="access_token_secret",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Access Token Secret"
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="consumer_key",
            field=models.CharField(blank=True, max_length=255, verbose_name="API Key"),
        ),
        migrations.AlterField(
            model_name="account",
            name="consumer_secret",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="API Key Secret"
            ),
        ),
    ]



import taggit.managers
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("taggit", "0001_initial"),
        ("pinboard", "0007_auto_20150724_1957"),
    ]

    operations = [
        migrations.AddField(
            model_name="bookmark",
            name="tags",
            field=taggit.managers.TaggableManager(
                to="taggit.Tag",
                through="taggit.TaggedItem",
                help_text="A comma-separated list of tags.",
                verbose_name="Tags",
            ),
        ),
    ]



from django.conf import settings

# Creating all the defaults for settings.
# In our code, if we want to use a DITTO_TWITTER_* setting we should import
# from here, not django.conf.settings.

TWITTER_DIR_BASE = getattr(settings, "DITTO_TWITTER_DIR_BASE", "twitter")

TWITTER_USE_LOCAL_MEDIA = getattr(settings, "DITTO_TWITTER_USE_LOCAL_MEDIA", False)



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="consumer_key",
            field=models.CharField(help_text=b"(API Key)", max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name="account",
            name="consumer_secret",
            field=models.CharField(
                help_text=b"(API Secret)", max_length=255, blank=True
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="screen_name",
            field=models.CharField(
                help_text=b"eg, 'philgyford'", unique=True, max_length=20, blank=True
            ),
        ),
    ]



import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Account",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("time_created", models.DateTimeField(auto_now_add=True)),
                ("time_modified", models.DateTimeField(auto_now=True)),
                (
                    "username",
                    models.CharField(
                        help_text=b"eg, 'philgyford'", unique=True, max_length=30
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        help_text=b"eg, 'https://pinboard.in/u:philgyford'",
                        max_length=255,
                    ),
                ),
                (
                    "api_token",
                    models.CharField(
                        help_text=b'From https://pinboard.in/settings/password eg, "philgyford:1234567890ABCDEFGHIJ"',  # noqa: E501
                        max_length=51,
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="Bookmark",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("time_created", models.DateTimeField(auto_now_add=True)),
                ("time_modified", models.DateTimeField(auto_now=True)),
                ("title", models.CharField(max_length=255, blank=True)),
                (
                    "permalink",
                    models.URLField(
                        help_text=b"URL of the item on the service's website.",
                        blank=True,
                    ),
                ),
                (
                    "summary",
                    models.TextField(
                        help_text=b"eg, First paragraph of a blog post, start of the description of a photo, all of a Tweet's text, etc.",  # noqa: E501
                        blank=True,
                    ),
                ),
                (
                    "is_private",
                    models.BooleanField(
                        default=False,
                        help_text=b"If True, this item should NOT be shown on public-facing pages.",  # noqa: E501
                    ),
                ),
                (
                    "fetch_time",
                    models.DateTimeField(
                        help_text=b"The time the Raw data was last fetched.",
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "raw",
                    models.TextField(
                        help_text=b"The raw JSON from the API.", blank=True
                    ),
                ),
                (
                    "url",
                    models.TextField(
                        unique=True, validators=[django.core.validators.URLValidator()]
                    ),
                ),
                ("post_time", models.DateTimeField()),
                (
                    "description",
                    models.TextField(
                        help_text=b"The 'extended' text description.", blank=True
                    ),
                ),
                ("to_read", models.BooleanField(default=False)),
                ("shared", models.BooleanField(default=True)),
                (
                    "account",
                    models.ForeignKey(to="pinboard.Account", on_delete=models.CASCADE),
                ),
            ],
            options={"abstract": False},
        ),
    ]



import os

from django.core.management.base import BaseCommand, CommandError

from ditto.twitter.ingest import Version1TweetIngester, Version2TweetIngester


class Command(BaseCommand):
    """Imports tweets from a downloaded archive of Tweets.
    Request your archive from https://twitter.com/settings/account

    Usage:
    ./manage.py import_tweets \
        --path=/Users/phil/Downloads/12552_dbeb4be9b8ff5f76d7d486c005cc21c9faa61f66
    """

    help = "Imports a complete history of tweets from a downloaded archive"

    def add_arguments(self, parser):
        parser.add_argument(
            "--path",
            action="store",
            default=False,
            help="Path to the directory that is the archive",
        )

        parser.add_argument(
            "--archive-version",
            action="store",
            default=None,
            help="v1 or v2 (default). Which format of archives to import from.",
        )

    def handle(self, *args, **options):
        # Location of the directory holding the tweet JSON files within the
        # archive:

        ingester_class = None

        # For v2, the default:
        # Where the JS files are:
        subpath = "/data"
        ingester_class = Version2TweetIngester

        if options["archive_version"]:
            if options["archive_version"] == "v1":
                # Where the JS files are:
                subpath = "/data/js/tweets"
                ingester_class = Version1TweetIngester

            elif options["archive_version"] != "v2":
                msg = f"version should be v1 or v2, not '{options['archive_version']}"
                raise CommandError(msg)

        if options["path"]:
            if os.path.isdir(options["path"]):
                js_dir = f"{options['path']}{subpath}"
                if os.path.isdir(js_dir):
                    result = ingester_class().ingest(directory=js_dir)
                else:
                    msg = (
                        f"Expected to find a directory at '{js_dir}' "
                        "containing .js file(s)"
                    )
                    raise CommandError(msg)
            else:
                msg = f"Can't find a directory at '{options['path']}'"
                raise CommandError(msg)
        else:
            msg = (
                "Specify the location of the archive directory, "
                "e.g. --path=/path/to/twitter-2022-01-31-abcdef123456"
            )
            raise CommandError(msg)

        if options.get("verbosity", 1) > 0:
            if result["success"]:
                tweetnoun = "tweet" if result["tweets"] == 1 else "tweets"
                filenoun = "file" if result["files"] == 1 else "files"
                mediafilenoun = "file" if result["media"] == 1 else "files"

                self.stdout.write(
                    f"Imported {result['tweets']} {tweetnoun} from "
                    f"{result['files']} {filenoun}, "
                    f"and {result['media']} media {mediafilenoun}"
                )
            else:
                self.stderr.write(f"Failed to import tweets: {result['messages'][0]}")



from django.apps import AppConfig


class DittoLastfmConfig(AppConfig):
    name = "ditto.lastfm"
    verbose_name = "Ditto Last.fm"

    # Maintain pre Django 3.2 default behaviour:
    default_auto_field = "django.db.models.AutoField"



from ditto.flickr.fetch.multifetchers import RecentPhotosMultiAccountFetcher

from . import FetchPhotosCommand


class Command(FetchPhotosCommand):
    """Fetches photos from Flickr.

    For all accounts:
        ./manage.py fetch_flickr_photos --days=3
        ./manage.py fetch_flickr_photos --days=all
        ./manage.py fetch_flickr_photos --start=2001-01-17
        ./manage.py fetch_flickr_photos --end=2003-11-10
        ./manage.py fetch_flickr_photos --start=2001-01-17 --end=2003-11-10

    For one account:
        ./manage.py fetch_flickr_photos --account=35034346050@N01 --days=3
        ./manage.py fetch_flickr_photos --account=35034346050@N01 --days=all
        ./manage.py fetch_flickr_photos --account=35034346050@N01
            --start=2001-01-17 --end=2003-11-10
    """

    help = "Fetches recent or all photos for one or all Flickr Accounts"

    days_help = 'Fetches the most recent or all Photos, eg "3" or "all".'

    start_help = (
        "Fetch photos taken on or after a date in YYYY-MM-DD format. "
        "Cannot be used with --days, can be combined with --end."
    )

    end_help = (
        "Fetch photos on or before a date in YYYY-MM-DD format. "
        "Cannot be used with --days, can be combined with --start."
    )

    def fetch_photos(self, nsid, days=None, start=None, end=None):
        return RecentPhotosMultiAccountFetcher(nsid=nsid).fetch(
            days=days, start=start, end=end
        )



# Generated by Django 5.1.5 on 2025-02-05 13:18

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0027_alter_bookmarktag_slug'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bookmark',
            name='url',
            field=models.URLField(max_length=1000, validators=[django.core.validators.URLValidator()], verbose_name='URL'),
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='url_hash',
            field=models.CharField(blank=True, db_index=True, help_text="Slug in the Bookmark's local URL.", max_length=12, verbose_name='URL hash'),
        ),
    ]



import datetime

import factory

from ditto.core.utils import datetime_now

from . import models


class AccountFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Account

    username = factory.Sequence(lambda n: f"user{n}")
    url = factory.LazyAttribute(lambda obj: f"https://pinboard.com/{obj.username}")
    api_token = factory.LazyAttribute(lambda obj: f"{obj.username}:123ABC")


class BookmarkFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Bookmark

    # DittoItem properties:
    title = factory.Sequence(lambda n: f"A Title {n}")
    is_private = False

    # Bookmark properties:
    account = factory.SubFactory(AccountFactory)
    url = factory.Sequence(lambda n: f"http://www.example.com/{n}")

    post_time = factory.LazyAttribute(
        lambda o: datetime_now() - datetime.timedelta(hours=1)
    )
    description = factory.Sequence(lambda n: f"A description of {n}")
    to_read = False

    # Leave tags empty by default.



from django.http import Http404
from django.utils.translation import gettext as _
from django.views.generic import DetailView, ListView
from django.views.generic.detail import SingleObjectMixin

from ditto.core.views import PaginatedListView

from .models import Account, Bookmark, BookmarkTag


class SingleAccountMixin(SingleObjectMixin):
    "For views which list bookmarks and also need an Account object."

    slug_field = "username"
    slug_url_kwarg = "username"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=Account.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["account"] = self.object
        return context


class HomeView(PaginatedListView):
    "List all recent Bookmarks and all Accounts"

    template_name = "pinboard/home.html"
    queryset = Bookmark.public_objects.all().prefetch_related("account")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["account_list"] = Account.objects.all()
        return context


class ToreadListView(PaginatedListView):
    template_name = "pinboard/toread_list.html"
    queryset = Bookmark.public_toread_objects.all().prefetch_related("account")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["account_list"] = Account.objects.all()
        return context


class AccountDetailView(SingleAccountMixin, PaginatedListView):
    "A single Pinboard Account and its Bookmarks."

    template_name = "pinboard/account_detail.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["bookmark_list"] = context["object_list"]
        return context

    def get_queryset(self):
        "Show all the public Bookmarks associated with this account."
        return Bookmark.public_objects.filter(account=self.object).prefetch_related(
            "account"
        )


class AccountToreadView(SingleAccountMixin, PaginatedListView):
    "A single Pinboard Account and its 'to read' Bookmarks."

    template_name = "pinboard/account_toread.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["bookmark_list"] = context["object_list"]
        return context

    def get_queryset(self):
        "Show all the public Bookmarks associated with this account."
        return Bookmark.public_toread_objects.filter(
            account=self.object
        ).prefetch_related("account")


class BookmarkDetailView(DetailView):
    "A single Bookmark, from one Account"

    model = Bookmark
    # Only display public bookmarks; private ones will 404.
    queryset = Bookmark.public_objects.all()
    slug_field = "url_hash"
    slug_url_kwarg = "hash"


class TagListView(ListView):
    template_name = "pinboard/tag_list.html"
    context_object_name = "tag_list"

    def get_queryset(self):
        return Bookmark.tags.most_common()[:100]

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["account_list"] = Account.objects.all()
        return context


class TagDetailView(SingleObjectMixin, PaginatedListView):
    "All Bookmarks with a certain tag from all Accounts"

    template_name = "pinboard/tag_detail.html"
    allow_empty = False

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=BookmarkTag.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["tag"] = self.object
        context["account_list"] = Account.objects.all()
        context["bookmark_list"] = context["object_list"]
        return context

    def get_queryset(self):
        "Show all the public Bookmarks associated with this tag."
        return Bookmark.public_objects.filter(
            tags__slug__in=[self.object.slug]
        ).prefetch_related("account")


class AccountTagDetailView(SingleAccountMixin, PaginatedListView):
    "All Bookmarks with a certain Tag from one Account"

    template_name = "pinboard/account_tag_detail.html"
    allow_empty = False

    def get(self, request, *args, **kwargs):
        self.tag_object = self.get_tag_object()
        return super().get(request, *args, **kwargs)

    def get_tag_object(self):
        """Custom method for fetching the Tag."""
        try:
            obj = BookmarkTag.objects.get(slug=self.kwargs["tag_slug"])
        except BookmarkTag.DoesNotExist as err:
            raise Http404(_("No Tags found matching the query")) from err
        return obj

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["tag"] = self.tag_object
        context["bookmark_list"] = context["object_list"]
        return context

    def get_queryset(self):
        """Show all the public Bookmarks associated with this account."""
        return Bookmark.public_objects.filter(
            account=self.object, tags__slug__in=[self.kwargs["tag_slug"]]
        ).prefetch_related("account")



from datetime import datetime, timezone
from datetime import time as datetime_time

from django import template
from django.utils.html import format_html

from ditto.core.utils import get_annual_item_counts
from ditto.flickr.models import Photo, Photoset

register = template.Library()


@register.simple_tag
def recent_photos(nsid=None, limit=10):
    """Returns a QuerySet of recent public Photos, in reverse-chronological
    order.

    Keyword arguments:
    nsid -- A Flickr user's NSID. If not supplied, we fetch
                    Photos for all Flickr users that have Accounts.
    limit -- Maximum number to fetch. Default is 10.
    """
    photos = Photo.public_photo_objects.all()
    if nsid is not None:
        photos = photos.filter(user__nsid=nsid)
    photos = photos.prefetch_related("user")
    return photos[:limit]


@register.simple_tag
def day_photos(date, nsid=None, time="post_time"):
    """Returns a QuerySet of public Photos posted on a specific date.

    Arguments:
    date -- A date object.

    Keyword arguments:
    nsid -- A Flickr user's NSID. If not supplied, we fetch
                    Photos for all Flickr users that have Accounts.
    time -- A string, either 'post_time' (default) or 'taken_time'.
    """
    if time not in ["post_time", "taken_time"]:
        msg = f"`time` must be either 'post_time' or 'taken_time', not '{time}'."
        raise ValueError(msg)

    start = datetime.combine(date, datetime_time.min).replace(tzinfo=timezone.utc)
    end = datetime.combine(date, datetime_time.max).replace(tzinfo=timezone.utc)
    photos = Photo.public_photo_objects

    if time == "taken_time":
        photos = photos.filter(taken_time__range=[start, end])
    else:
        photos = photos.filter(post_time__range=[start, end])

    if nsid is not None:
        photos = photos.filter(user__nsid=nsid)

    photos = photos.prefetch_related("user")
    return photos


@register.simple_tag
def photosets(nsid=None, limit=10):
    """Returns a QuerySet of recent Photosets.

    Keyword arguments:
    nsid -- A Flickr user's NSID. If not supplied, we fetch
                    Photosets for all Flickr users that have Accounts.
    limit -- Maximum number to fetch. Default is 10.
    """
    photosets = Photoset.objects.all()
    if nsid is not None:
        photosets = photosets.filter(user__nsid=nsid)
    return photosets.prefetch_related("primary_photo", "user")[:limit]


@register.simple_tag
def photo_license(n):
    """Returns the text value of the Photo's license, indicated by the number n.
    Will probably be an HTML link to more info.
    """
    licenses = {x: y for x, y in Photo.LICENSES}

    if n in licenses:
        if n in Photo.LICENSE_URLS and Photo.LICENSE_URLS[n] != "":
            return format_html(
                '<a href="{}" title="More about permissions">{}</a>',
                Photo.LICENSE_URLS[n],
                licenses[n],
            )
        else:
            return licenses[n]
    else:
        return "[missing]"


@register.simple_tag
def annual_photo_counts(nsid=None, count_by="post_time"):
    """
    Get the number of public Photos per year.
    Returns a list of dicts, sorted by year, like:
        [ {'year': 2015, 'count': 1234}, {'year': 2016, 'count': 9876} ]

    Keyword arguments:
    nsid -- A Flickr user's NSID or None (for Photos by all Users).
    count_by -- A string, either 'post_time' (default) or 'taken_time'.
    """

    if count_by not in ["post_time", "taken_time"]:
        msg = (
            f"`count_by` must be either 'post_time' or 'taken_time', not '{count_by}'."
        )
        raise ValueError(msg)

    qs = Photo.public_photo_objects

    if nsid is not None:
        qs = qs.filter(user__nsid=nsid)

    field_name = "taken_year" if count_by == "taken_time" else "post_year"

    return get_annual_item_counts(qs, field_name)



# Generated by Django 4.0.3 on 2022-03-23 15:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0027_alter_user_photos_url_alter_user_profile_url"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="iconfarm",
            field=models.PositiveIntegerField(default=0),
        ),
    ]



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0002_auto_20150626_1521"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="bookmark",
            unique_together={("account", "url")},
        ),
    ]



# Generated by Django 1.10.1 on 2016-11-25 11:04

from django.db import migrations, models


def set_taken_year(apps, schema_editor):
    """
    Sets the `taken_year` value on every Photo.
    """
    Photo = apps.get_model("flickr", "Photo")
    for row in Photo.objects.all():
        if row.taken_time:
            row.taken_year = row.taken_time.year
            row.save()


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0021_photo_post_year"),
    ]

    operations = [
        migrations.AddField(
            model_name="photo",
            name="taken_year",
            field=models.PositiveSmallIntegerField(
                blank=True,
                db_index=True,
                help_text="Set automatically on save",
                null=True,
            ),
        ),
        migrations.RunPython(set_taken_year, reverse_code=migrations.RunPython.noop),
    ]



from ditto.flickr.fetch.multifetchers import PhotosetsMultiAccountFetcher

from . import FetchCommand


class Command(FetchCommand):
    """Fetches photosets from Flickr

    For all accounts:
        ./manage.py fetch_flickr_photosets

    For one account:
        ./manage.py fetch_flickr_photosets --account=35034346050@N01
    """

    # What we're fetching:
    singular_noun = "Photoset"
    plural_noun = "Photosets"

    help = (
        "Fetches all photosets for one or all Flickr Accounts "
        "(photo data must already be fetched)."
    )

    def handle(self, *args, **options):
        # We might be fetching for a specific account or all (None).
        nsid = options["account"] if options["account"] else None

        results = PhotosetsMultiAccountFetcher(nsid=nsid).fetch()

        self.output_results(results, options.get("verbosity", 1))



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0007_auto_20150807_1432"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="user",
            name="twitter_id_str",
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0013_user_favorites"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tweet",
            name="favorite_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"Approximately how many times this had been favorited when fetched",  # noqa: E501
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="tweet",
            name="retweet_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"Number of times this had been retweeted when fetched",
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="url",
            field=models.URLField(
                default=b"",
                help_text=b"A URL provided by the user as part of their profile",
                blank=True,
            ),
        ),
    ]



from django.urls import path, re_path

from . import views

app_name = "lastfm"


# The pattern for matching an Album/Artist/Track slug:
slug_chars = r"[\w.,:;=@&+%()$!°’~-]+"


urlpatterns = [
    path("", view=views.HomeView.as_view(), name="home"),
    path("library/", view=views.ScrobbleListView.as_view(), name="scrobble_list"),
    path(
        "library/albums/",
        view=views.AlbumListView.as_view(),
        name="album_list",
    ),
    path(
        "library/artists/",
        view=views.ArtistListView.as_view(),
        name="artist_list",
    ),
    path(
        "library/tracks/",
        view=views.TrackListView.as_view(),
        name="track_list",
    ),
    re_path(
        rf"^music/(?P<artist_slug>{slug_chars})/$",
        view=views.ArtistDetailView.as_view(),
        name="artist_detail",
    ),
    re_path(
        rf"^music/(?P<artist_slug>{slug_chars})/\+albums/$",
        view=views.ArtistAlbumsView.as_view(),
        name="artist_albums",
    ),
    re_path(
        rf"^music/(?P<artist_slug>{slug_chars})/(?P<album_slug>{slug_chars})/$",
        view=views.AlbumDetailView.as_view(),
        name="album_detail",
    ),
    re_path(
        rf"^music/(?P<artist_slug>{slug_chars})/_/(?P<track_slug>{slug_chars})/$",
        view=views.TrackDetailView.as_view(),
        name="track_detail",
    ),
    # User pages.
    re_path(
        r"^user/(?P<username>[a-z0-9]+)/$",
        view=views.UserDetailView.as_view(),
        name="user_detail",
    ),
    re_path(
        r"^user/(?P<username>[a-z0-9]+)/library/$",
        view=views.UserScrobbleListView.as_view(),
        name="user_scrobble_list",
    ),
    re_path(
        r"^user/(?P<username>[a-z0-9]+)/library/albums/$",
        view=views.UserAlbumListView.as_view(),
        name="user_album_list",
    ),
    re_path(
        r"^user/(?P<username>[a-z0-9]+)/library/artists/$",
        view=views.UserArtistListView.as_view(),
        name="user_artist_list",
    ),
    re_path(
        r"^user/(?P<username>[a-z0-9]+)/library/tracks/$",
        view=views.UserTrackListView.as_view(),
        name="user_track_list",
    ),
]



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0026_auto_20151110_1131"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="user",
            name="profile_image_url",
        ),
    ]



import contextlib
import os
import re
import shutil

import requests


class DownloadException(Exception):  # noqa: N818
    pass


class FileDownloader:
    """
    For downloading a file from a URL and saving it into /tmp/.

    Use like:
        from ditto.core.utils.downloader import filedownloader
        filepath = filedownloader.download(my_url, ['image/jpg'])

    filepath would be like '/tmp/image.jpg'
    """

    path = "/tmp/"

    def download(self, url, acceptable_content_types):
        """
        Downloads a file from a URL and saves it into /tmp/.
        Returns the filepath.

        Expects:
            url -- The URL of the file to fetch.
            acceptable_content_types -- A list of MIME types the request must
                match. eg:['image/jpeg', 'image/jpg', 'image/png', 'image/gif']

        Raises DownloadException if something goes wrong.
        """
        try:
            # From http://stackoverflow.com/a/13137873/250962
            r = requests.get(url, stream=True)
            if r.status_code == 200:
                try:
                    if r.headers["Content-Type"] in acceptable_content_types:
                        # Where we'll temporarily save the file:
                        filename = self.make_filename(url, r.headers)
                        filepath = f"{self.path}{filename}"
                        # Save the file there:
                        with open(filepath, "wb") as f:
                            r.raw.decode_content = True
                            shutil.copyfileobj(r.raw, f)
                        return filepath

                    else:
                        msg = "Invalid content type ({}) when fetching {}".format(
                            r.headers["content_type"], url
                        )
                        raise DownloadException(msg)
                except KeyError as err:
                    msg = f"No content_type headers found when fetching {url}"
                    raise DownloadException(msg) from err
            else:
                msg = f"Got status code {r.status_code} when fetching {url}"
                raise DownloadException(msg)
        except requests.exceptions.RequestException as err:
            msg = f"Something when wrong when fetching {url}: {err}"
            raise DownloadException(msg) from err

    def make_filename(self, url, headers=None):
        """
        Find the filename of a downloaded file.
        Returns a string.

        url -- The URL of the file that's been downloaded.
        headers -- A dict of response headers from requesting the URL.

        url will probably end in something like 'filename.jpg'.
        If not, we'll try and use the filename from the Content-Disposition
        header.
        This is the case for Videos we download from Flickr.
        """
        headers = {} if headers is None else headers

        # Should work for photos:
        filename = os.path.basename(url)

        if filename == "":
            # Probably a Flickr video, so we have to get the filename from
            # headers:
            try:
                # Could be like 'attachment; filename=26897200312.avi'
                headers["Content-Disposition"]
                m = re.search(r"filename\=(.*?)$", headers["Content-Disposition"])
                with contextlib.suppress(AttributeError, IndexError):
                    filename = m.group(1)
            except KeyError:
                pass

        return filename


filedownloader = FileDownloader()



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0023_media"),
    ]

    operations = [
        migrations.AddField(
            model_name="media",
            name="dash_url",
            field=models.URLField(null=True, verbose_name="MPEG-DASH URL", blank=True),
        ),
        migrations.AddField(
            model_name="media",
            name="webm_bitrate",
            field=models.PositiveIntegerField(
                null=True, verbose_name="WebM Bitrate", blank=True
            ),
        ),
        migrations.AddField(
            model_name="media",
            name="xmpeg_url",
            field=models.URLField(null=True, verbose_name="X-MPEG URL", blank=True),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0010_remove_tweet_text"),
    ]

    operations = [
        migrations.AddField(
            model_name="tweet",
            name="text",
            field=models.TextField(default="", max_length=140),
            preserve_default=False,
        ),
    ]



from django.contrib import admin
from django.db import models
from django.forms import Textarea, TextInput
from taggit.forms import TagWidget
from taggit.managers import TaggableManager

from ditto.core.admin import DittoItemModelAdmin

from .models import Account, Bookmark


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = (
        "username",
        "is_active",
        "time_created",
        "time_modified",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "username",
                    "url",
                    "is_active",
                )
            },
        ),
        (
            "API",
            {
                "fields": ("api_token",),
                "description": (
                    "Your API Token can be found at "
                    '<a href="https://pinboard.in/settings/password">'
                    "pinboard.in/settings/password</a>"
                ),
            },
        ),
        (
            "Data",
            {
                "fields": (
                    "time_created",
                    "time_modified",
                )
            },
        ),
    )
    readonly_fields = (
        "time_created",
        "time_modified",
    )


@admin.register(Bookmark)
class BookmarkAdmin(DittoItemModelAdmin):
    list_display = (
        "title",
        "post_time",
        "account",
    )
    list_filter = (
        "post_time",
        "is_private",
        "to_read",
        "account",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "account",
                    "url",
                    "title",
                    "description",
                    "tags",
                    ("is_private", "to_read"),
                    "post_time",
                )
            },
        ),
        (
            "Raw data, times, etc.",
            {
                "classes": ("collapse",),
                "fields": (
                    "summary",
                    "url_hash",
                    "raw",
                    "post_year_str",
                    "fetch_time",
                    "time_created",
                    "time_modified",
                ),
            },
        ),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {"widget": TextInput(attrs={"class": "vLargeTextField"})},
        # Reduce the number of rows; similar to Pinboard's description field.
        models.TextField: {
            "widget": Textarea(
                attrs={"class": "vLargeTextField", "cols": 40, "rows": 4}
            )
        },
        # Make the input full-width.
        TaggableManager: {"widget": TagWidget(attrs={"class": "vLargeTextField"})},
    }

    readonly_fields = (
        "post_year_str",
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
    )
    search_fields = (
        "title",
        "url",
        "description",
        "tags__name",
    )

    def get_changeform_initial_data(self, request):
        """
        If there's only one Account, set it as the initial value for
        the account field.
        (Unless it's already set from the GET values.)
        """
        initial_data = super().get_changeform_initial_data(request)
        if Account.objects.count() == 1 and "account" not in initial_data:
            initial_data["account"] = str(Account.objects.first().pk)
        return initial_data



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0024_auto_20151104_1157"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="media",
            options={
                "verbose_name": "Media item",
                "ordering": ["time_created"],
                "verbose_name_plural": "Media items",
            },
        ),
        migrations.AlterField(
            model_name="tweet",
            name="twitter_id",
            field=models.BigIntegerField(db_index=True, unique=True),
        ),
    ]



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0009_account_last_fetch_id"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="tweet",
            name="text",
        ),
    ]



# Generated by Django 1.10.1 on 2016-10-11 11:07

import django.db.models.deletion
from django.db import migrations, models

import ditto.core.models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Account",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                (
                    "username",
                    models.CharField(help_text="eg, 'rj'", max_length=30, unique=True),
                ),
                (
                    "realname",
                    models.CharField(help_text="eg, 'Richard Jones'", max_length=30),
                ),
                (
                    "api_key",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="API Key"
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="If false, new scrobbles won't be fetched.",
                    ),
                ),
            ],
            options={"ordering": ["username"]},
        ),
        migrations.CreateModel(
            name="Album",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                ("name", models.TextField()),
                ("slug", models.TextField(db_index=True)),
                (
                    "mbid",
                    models.CharField(
                        blank=True,
                        help_text="MusicBrainz Identifier",
                        max_length=36,
                        verbose_name="MBID",
                    ),
                ),
            ],
            options={"ordering": ["name"]},
        ),
        migrations.CreateModel(
            name="Artist",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("slug", models.TextField(unique=True)),
                (
                    "mbid",
                    models.CharField(
                        blank=True,
                        help_text="MusicBrainz Identifier",
                        max_length=36,
                        verbose_name="MBID",
                    ),
                ),
            ],
            options={"ordering": ["name"]},
        ),
        migrations.CreateModel(
            name="Scrobble",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                ("title", models.CharField(blank=True, max_length=255)),
                (
                    "permalink",
                    models.URLField(
                        blank=True,
                        help_text="URL of the item on the service's website.",
                    ),
                ),
                (
                    "summary",
                    models.CharField(
                        blank=True,
                        help_text="eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.",  # noqa: E501
                        max_length=255,
                    ),
                ),
                (
                    "is_private",
                    models.BooleanField(
                        default=False,
                        help_text="If true, this item will not be shown on public-facing pages.",  # noqa: E501
                    ),
                ),
                (
                    "fetch_time",
                    models.DateTimeField(
                        blank=True,
                        help_text="The time the item's data was last fetched.",
                        null=True,
                    ),
                ),
                (
                    "post_time",
                    models.DateTimeField(
                        blank=True,
                        help_text="The time the item was originally posted/created on its service.",  # noqa: E501
                        null=True,
                    ),
                ),
                (
                    "latitude",
                    models.DecimalField(
                        blank=True, decimal_places=6, max_digits=9, null=True
                    ),
                ),
                (
                    "longitude",
                    models.DecimalField(
                        blank=True, decimal_places=6, max_digits=9, null=True
                    ),
                ),
                (
                    "raw",
                    models.TextField(
                        blank=True, help_text="eg, the raw JSON from the API."
                    ),
                ),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="lastfm.Account"
                    ),
                ),
                (
                    "album",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="scrobbles",
                        to="lastfm.Album",
                    ),
                ),
                (
                    "artist",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="scrobbles",
                        to="lastfm.Artist",
                    ),
                ),
            ],
            options={"ordering": ["-post_time"]},
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="Track",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                ("name", models.TextField()),
                ("slug", models.TextField(db_index=True)),
                (
                    "mbid",
                    models.CharField(
                        blank=True,
                        help_text="MusicBrainz Identifier",
                        max_length=36,
                        verbose_name="MBID",
                    ),
                ),
                (
                    "artist",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="tracks",
                        to="lastfm.Artist",
                    ),
                ),
            ],
            options={"ordering": ["name"]},
        ),
        migrations.AddField(
            model_name="scrobble",
            name="track",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="scrobbles",
                to="lastfm.Track",
            ),
        ),
        migrations.AddField(
            model_name="album",
            name="artist",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="albums",
                to="lastfm.Artist",
            ),
        ),
    ]



# Generated by Django 1.9.5 on 2016-05-25 10:11

from django.db import migrations, models

import ditto.twitter.models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0036_auto_20160505_1156"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="avatar",
            field=models.ImageField(
                blank=True,
                default="",
                upload_to=ditto.twitter.models.User.avatar_upload_path,
            ),
        ),
    ]



from django.conf import settings

# Creating all the defaults for settings.
# In our code, if we want to use a DITTO_* setting we should import
# from here, not django.conf.settings.

# e.g. "07:34"
CORE_TIME_FORMAT = getattr(settings, "DITTO_CORE_TIME_FORMAT", "%H:%M")

# e.g. "8 Apr 2018"
CORE_DATE_FORMAT = getattr(settings, "DITTO_CORE_DATE_FORMAT", "%-d %b %Y")

# e.g. "07:34 on 8 Apr 2018"
CORE_DATETIME_FORMAT = getattr(
    settings, "DITTO_CORE_DATETIME_FORMAT", "[time] on [date]"
)

# e.g. "2018"
CORE_DATE_YEAR_FORMAT = getattr(settings, "DITTO_CORE_DATE_YEAR_FORMAT", "%Y")

# e.g. "Apr 2018"
CORE_DATE_YEAR_MONTH_FORMAT = getattr(
    settings, "DITTO_CORE_DATE_YEAR_MONTH_FORMAT", "%b %Y"
)



import datetime

import factory

from ditto.core.utils import datetime_now

from . import models


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.User

    twitter_id = factory.Sequence(lambda n: (n * 10000))
    screen_name = factory.Sequence(lambda n: f"user{n}")
    name = factory.Sequence(lambda n: f"User Name {n}")

    created_at = factory.LazyAttribute(
        lambda o: datetime_now() - datetime.timedelta(weeks=52)
    )

    description = factory.Sequence(lambda n: f"A description {n}")
    is_private = False
    fetch_time = datetime_now()

    avatar = factory.django.ImageField(filename="my_avatar.jpg")


class AccountFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Account

    user = factory.SubFactory(
        UserFactory,
        twitter_id=factory.Sequence(lambda n: n),
        screen_name=factory.Sequence(lambda n: f"user{n}"),
    )


class AccountWithCredentialsFactory(AccountFactory):
    """We only want to add these when we're going to be testing the
    fetching of API data for the Account's user.
    """

    consumer_key = "TESTCONSUMERKEY"
    consumer_secret = "TESTCONSUMERSECRET"
    access_token = factory.Sequence(lambda n: f"{n}-TESTACCESSTOKEN")
    access_token_secret = "TESTACCESSTOKENSECRET"


class TweetFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = models.Tweet

    user = factory.SubFactory(UserFactory)
    text = factory.Sequence(lambda n: f"The text of tweet {n}")
    twitter_id = factory.Sequence(lambda n: (n * 10000000))
    fetch_time = datetime_now()

    post_time = factory.LazyAttribute(
        lambda o: datetime_now() - datetime.timedelta(weeks=4)
    )
    source = "web"


class MediaFactory(factory.django.DjangoModelFactory):
    "Parent class for the photo, video and gif factories."

    class Meta:
        model = models.Media

    twitter_id = factory.Sequence(lambda n: (n * 10000000))
    large_w = 640
    large_h = 360
    medium_w = 600
    medium_h = 338
    small_w = 340
    small_h = 191
    thumb_w = 150
    thumb_h = 150

    image_file = factory.django.ImageField(filename="example.jpg")
    mp4_file = factory.django.FileField(filename="example.mp4")

    @factory.post_generation
    def tweets(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for tweet in extracted:
                self.tweets.add(tweet)


class PhotoFactory(MediaFactory):
    media_type = "photo"

    large_w = 938
    large_h = 397
    medium_w = 600
    medium_h = 253
    small_w = 340
    small_h = 143
    thumb_w = 150
    thumb_h = 150

    image_url = factory.Sequence(lambda n: f"http://pbs.twimg.com/media/{n}.jpg")
    image_file = factory.django.ImageField(filename="example.jpg")


class VideoFactory(MediaFactory):
    media_type = "video"

    image_url = factory.Sequence(
        lambda n: f"http://pbs.twimg.com/ext_tw_video_thumb/{n}/pu/img/{n}.jpg"
    )

    xmpeg_url = factory.Sequence(
        lambda n: f"https://video.twimg.com/ext_tw_video/{n}/pu/pl/{n}.m3u8"
    )
    dash_url = factory.Sequence(
        lambda n: f"https://video.twimg.com/ext_tw_video/{n}/pu/pl/{n}.mpd"
    )

    aspect_ratio = "16:9"
    duration = 20000

    image_file = factory.django.ImageField(filename="example.jpg")


class AnimatedGifFactory(MediaFactory):
    media_type = "animated_gif"

    image_url = factory.Sequence(
        lambda n: f"http://pbs.twimg.com/ext_tw_video_thumb/{n}/pu/img/{n}.jpg"
    )

    mp4_url = factory.Sequence(lambda n: f"https://pbs.twimg.com/tweet_video/{n}.mp4")

    aspect_ratio = "16:9"

    image_file = factory.django.ImageField(filename="example.jpg")
    mp4_file = factory.django.FileField(filename="example.mp4")



# Generated by Django 5.1.5 on 2025-02-05 15:12

import taggit.managers
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0029_alter_bookmark_post_time'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bookmark',
            name='tags',
            field=taggit.managers.TaggableManager(blank=True, help_text='Separated by spaces. Private tags start with a period.', through='pinboard.TaggedBookmark', to='pinboard.BookmarkTag', verbose_name='Tags'),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="photo",
            name="exif_iso",
            field=models.IntegerField(null=True, help_text="eg, '100'.", blank=True),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0013_auto_20151026_1546"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="bookmarktag",
            options={"verbose_name_plural": "Tags", "verbose_name": "Tag"},
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="is_private",
            field=models.BooleanField(
                default=False,
                help_text=(
                    "If true, this item will not be " "shown on public-facing pages."
                ),
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0004_auto_20150730_1116"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tweet",
            name="favorite_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"Approximately how many times this has been favorited",
            ),
        ),
        migrations.AlterField(
            model_name="tweet",
            name="retweet_count",
            field=models.PositiveIntegerField(
                default=0, help_text=b"Number of times this has been retweeted"
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0009_auto_20150729_1056"),
    ]

    operations = [
        migrations.AddField(
            model_name="bookmark",
            name="latitude",
            field=models.DecimalField(
                null=True, max_digits=12, decimal_places=9, blank=True
            ),
        ),
        migrations.AddField(
            model_name="bookmark",
            name="longitude",
            field=models.DecimalField(
                null=True, max_digits=12, decimal_places=9, blank=True
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0017_auto_20160413_0906"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bookmark",
            name="latitude",
            field=models.DecimalField(
                decimal_places=6, blank=True, null=True, max_digits=9
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="longitude",
            field=models.DecimalField(
                decimal_places=6, blank=True, null=True, max_digits=9
            ),
        ),
    ]



# Generated by Django 4.0.3 on 2022-03-23 14:48

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taggit", "0004_alter_taggeditem_content_type_alter_taggeditem_tag"),
        ("flickr", "0025_backfill_photo_sizes"),
    ]

    operations = [
        migrations.AlterField(
            model_name="taggedphoto",
            name="content_object",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_items",
                to="flickr.photo",
            ),
        ),
        migrations.AlterField(
            model_name="taggedphoto",
            name="tag",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(app_label)s_%(class)s_items",
                to="taggit.tag",
            ),
        ),
    ]



from django.urls import path, re_path

from . import views

app_name = "ditto"

urlpatterns = [
    path("", view=views.HomeView.as_view(), name="home"),
    # path(
    # "tags",
    # view=views.TagListView.as_view(),
    # name='tag_list'
    # ),
    # re_path(
    # r"^tags/(?P<slug>[^/]+)$",
    # view=views.TagDetailView.as_view(),
    # name='tag_detail'
    # ),
    re_path(
        # /2016/04/18/twitter/favorites
        r"^(?P<year>[0-9]{4})/(?P<month>[0-9]{2})/(?P<day>[0-9]{2})(?:/(?P<app>[a-z]+))?(?:/(?P<variety>[a-z\/]+|))?$",
        view=views.DayArchiveView.as_view(),
        name="day_archive",
    ),
]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0029_auto_20151110_1308"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="consumer_key",
            field=models.CharField(
                help_text="(API Key) From https://apps.twitter.com",
                blank=True,
                max_length=255,
            ),
        ),
    ]



# Generated by Django 4.0.3 on 2022-03-24 10:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0028_alter_user_iconfarm"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="user",
            options={"ordering": ["realname", "username"]},
        ),
        migrations.AlterField(
            model_name="user",
            name="realname",
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



# Generated by Django 1.9.5 on 2016-05-25 10:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0017_auto_20160524_1350"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="photos_url",
            field=models.URLField(max_length=255, verbose_name="Photos URL at Flickr"),
        ),
        migrations.AlterField(
            model_name="user",
            name="profile_url",
            field=models.URLField(max_length=255, verbose_name="Avatar URL on Flickr"),
        ),
    ]



# Generated by Django 1.9.5 on 2016-06-03 13:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0040_auto_20160603_1302"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="media",
            name="tweet",
        ),
    ]



# Generated by Django 3.1.3 on 2020-12-23 15:09

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("pinboard", "0024_bookmark_post_year"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bookmarktag",
            name="name",
            field=models.CharField(max_length=100, unique=True, verbose_name="name"),
        ),
        migrations.AlterField(
            model_name="bookmarktag",
            name="slug",
            field=models.SlugField(max_length=100, unique=True, verbose_name="slug"),
        ),
        migrations.AlterField(
            model_name="taggedbookmark",
            name="content_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="pinboard_taggedbookmark_tagged_items",
                to="contenttypes.contenttype",
                verbose_name="content type",
            ),
        ),
        migrations.AlterField(
            model_name="taggedbookmark",
            name="object_id",
            field=models.IntegerField(db_index=True, verbose_name="object ID"),
        ),
    ]



from django.conf import settings

# Creating all the defaults for settings.
# In our code, if we want to use a DITTO_FLICKR_* setting we should import
# from here, not django.conf.settings.


FLICKR_DIR_BASE = getattr(settings, "DITTO_FLICKR_DIR_BASE", "flickr")

FLICKR_DIR_PHOTOS_FORMAT = getattr(
    settings, "DITTO_FLICKR_DIR_PHOTOS_FORMAT", "%Y/%m/%d"
)

FLICKR_USE_LOCAL_MEDIA = getattr(settings, "DITTO_FLICKR_USE_LOCAL_MEDIA", False)



import calendar
import json
import time
import urllib
from datetime import datetime, timedelta, timezone

import requests

from ditto import TITLE, VERSION
from ditto.core.utils import datetime_now

from .models import Account, Album, Artist, Scrobble, Track
from .utils import slugify_name

LASTFM_API_ENDPOINT = "http://ws.audioscrobbler.com/2.0/"


class FetchError(Exception):
    pass


class ScrobblesFetcher:
    """
    Fetches scrobbles from the API for one Account.

    Use like one of:
        fetcher = ScrobblesFetcher(account)

    And then one of these ('recent' is the default):
        results = fetcher.fetch(fetch_type='recent')
        results = fetcher.fetch(fetch_type='all')
        results = fetcher.fetch(fetch_type='days', days=3)
    """

    # How many scrobbles do we fetch per page of results?
    items_per_page = 200

    def __init__(self, account):
        # Will be an Account object, passed into init()
        self.account = None

        # We'll set this to a datetime if we're fetching scrobbles since x.
        self.min_datetime = None

        self.page_number = 1

        self.total_pages = 1

        self.results_count = 0

        # What we'll return:
        self.return_value = {"fetched": 0}

        if isinstance(account, Account):
            self.return_value["account"] = str(account)
        else:
            msg = "An Account object is required"
            raise ValueError(msg)

        if account.has_credentials():
            self.account = account
        else:
            self.return_value["success"] = False
            self.return_value["messages"] = ["Account has no API credentials"]

    def fetch(self, fetch_type="recent", days=None):
        """
        Fetch and save scrobbles.

        Keyword arguments:
        fetch_type -- 'all', 'days' or 'recent'. The latter will fetch
                      scrobbles since the most recent Scrobble we already have.
        days -- if fetch_type is 'days', this should be an integer.

        Returns a dict like:
            {'success': True, 'account': 'gyford', 'fetched': 47,}
        Or:
            {'success': False, 'account': 'gyford', 'messages': ['Oops..',],}
        """

        if self.account and self.account.is_active is False:
            self.return_value["success"] = False
            self.return_value["messages"] = [
                f"The Account {self.account.username} is currently marked as inactive."
            ]
            return self.return_value

        valid_fetch_types = ["all", "days", "recent"]
        if fetch_type not in valid_fetch_types:
            types_str = ", ".join(valid_fetch_types)
            msg = f"fetch_type should be one of {types_str}"
            raise ValueError(msg)

        if fetch_type == "days":
            try:
                test = days + 1  # noqa: F841
            except TypeError as err:
                msg = "days argument should be an integer"
                raise ValueError(msg) from err

            self.min_datetime = datetime_now() - timedelta(days=days)

        elif fetch_type == "recent":
            try:
                scrobble = Scrobble.objects.latest("post_time")
                self.min_datetime = scrobble.post_time
            except Scrobble.DoesNotExist:
                pass

        self._fetch_pages()

        if self._not_failed():
            self.return_value["success"] = True
            self.return_value["fetched"] = self.results_count

        return self.return_value

    def _fetch_pages(self):
        while self.page_number <= self.total_pages and self._not_failed():
            self._fetch_page()
            self.page_number += 1
            time.sleep(0.5)

    def _fetch_page(self):
        """
        Fetch a single page of results.
        Uses the value of self.page_number.
        """
        fetch_time = datetime_now()

        try:
            results = self._send_request()
        except FetchError as e:
            self.return_value["success"] = False
            self.return_value["messages"] = [str(e)]
            return

        for scrobble in results:
            if "date" in scrobble:
                # Don't save nowplaying scrobbles, that have no 'date'.
                self._save_scrobble(scrobble, fetch_time)
                self.results_count += 1

        return

    def _not_failed(self):
        """Has everything gone smoothly so far? ie, no failure registered?"""
        return (
            "success" not in self.return_value or self.return_value["success"] is True
        )

    def _api_method(self):
        "The name of the API method."
        return "user.getrecenttracks"

    def _api_args(self):
        "Returns a dict of args for the API call."
        args = {
            "user": self.account.username,
            "api_key": self.account.api_key,
            "format": "json",
            "method": self._api_method(),
            "page": self.page_number,
            "limit": self.items_per_page,
        }

        if self.min_datetime:
            # Turn our datetime object into a unix timestamp:
            args["from"] = calendar.timegm(self.min_datetime.timetuple())

        return args

    def _send_request(self):
        """
        Send a request to the Last.fm API.

        Raises FetchError if something goes wrong.
        Returns a list of results if all goes well.
        """
        query_string = urllib.parse.urlencode(self._api_args())

        url = f"{LASTFM_API_ENDPOINT}?{query_string}"

        try:
            response = requests.get(
                url,
                headers={"User-Agent": f"Mozilla/5.0 ({TITLE} v{VERSION})"},
            )
            response.raise_for_status()  # Raises an exception on HTTP error.
        except requests.exceptions.RequestException as err:
            msg = f"Error when fetching Scrobbles (page {self.page_number}): {err}"
            raise FetchError(msg) from err

        response.encoding = "utf-8"

        results = json.loads(response.text)

        if "error" in results:
            msg = "Error {} when fetching Scrobbles (page {}): {}".format(
                results["error"], self.page_number, results["message"]
            )
            raise FetchError(msg)

        # Set total number of pages first time round:
        attr = results["recenttracks"]["@attr"]
        if self.page_number == 1 and "totalPages" in attr:
            self.total_pages = int(attr["totalPages"])

        return results["recenttracks"]["track"]

    def _save_scrobble(self, scrobble, fetch_time):
        """
        Saves/updates a scrobble.

        Arguments:
        scrobble -- A dict of data from the Last.fm API.
        fetch_time -- Datetime of when the data was fetched.
        """
        artist_slug, track_slug = self._get_slugs(scrobble["url"])

        artist, created = Artist.objects.update_or_create(
            slug=artist_slug.lower(),
            defaults={
                "name": scrobble["artist"]["#text"],
                "original_slug": artist_slug,
                "mbid": scrobble["artist"]["mbid"],  # Might be "".
            },
        )

        track, created = Track.objects.update_or_create(
            slug=track_slug.lower(),
            artist=artist,
            defaults={
                "name": scrobble["name"],
                "original_slug": track_slug,
                "mbid": scrobble["mbid"],  # Might be "".
            },
        )

        if scrobble["album"]["#text"] == "":
            album = None
        else:
            # The API data doesn't provide a URL/slug for the album, so
            # we make our own:
            album_slug = slugify_name(scrobble["album"]["#text"])

            album, created = Album.objects.update_or_create(
                slug=album_slug.lower(),
                artist=artist,
                defaults={
                    "name": scrobble["album"]["#text"],
                    "original_slug": album_slug,
                    "mbid": scrobble["album"]["mbid"],  # Might be "".
                },
            )

        # Unixtime to datetime object:
        scrobble_time = datetime.fromtimestamp(
            int(scrobble["date"]["uts"]), tz=timezone.utc
        )

        scrobble_obj, created = Scrobble.objects.update_or_create(
            account=self.account,
            track=track,
            post_time=scrobble_time,
            defaults={
                "artist": artist,
                "raw": json.dumps(scrobble),
                "fetch_time": fetch_time,
                "album": album,
            },
        )

        return scrobble_obj

    def _get_slugs(self, scrobble_url):
        """
        Get the artist and track slugs from a scrobble's URL.
        The scrobble's URL is also the Track's URL.

        scrobble_url is like 'https://www.last.fm/music/Artist/_/Track'
        returns two strings, artist_slug and track_slug.
        """
        url = scrobble_url.rstrip("/")

        # Need to replace semicolons as urlparse() treats them (legitimately)
        # as alternatives to '&' as a query string separator, and so omits
        # anything after them.
        url = url.replace(";", "%3B")

        # www.last.fm/music/Artist/_/Track':
        url_path = urllib.parse.urlparse(url).path
        path_parts = url_path.split("/")

        artist_slug = path_parts[-3]  # 'Artist'
        track_slug = path_parts[-1]  # 'Track'

        # Put those naughty semicolons back in:
        artist_slug = artist_slug.replace("%3B", ";")
        track_slug = track_slug.replace("%3B", ";")

        return artist_slug, track_slug


class ScrobblesMultiAccountFetcher:
    """
    For fetching Scrobbles for ALL or ONE account(s).

    Usage example:
      results = ScrobblesMultiAccountFetcher().fetch(fetch_type='recent')

    Or:
      results = ScrobblesMultiAccountFetcher(username='bob').fetch(fetch_type='recent')

    results will be a list of dicts containing info about what was fetched (or
    went wrong) for each account.
    """

    # Will be a list of Account objects.
    accounts = []

    def __init__(self, username=None):
        """
        Gets all of the Accounts, or the single Account specified.

        username -- If username is set, we only use that Account, if active.
                    If it's not set, we use all active Accounts.
        """
        self.return_value = []

        if username is None:
            # Get all active Accounts.
            self.accounts = list(Account.objects.filter(is_active=True))
            if len(self.accounts) == 0:
                msg = "No active Accounts were found to fetch."
                raise FetchError(msg)
        else:
            # Find the Account associated with username.
            try:
                account = Account.objects.get(username=username)
            except Account.DoesNotExist as err:
                msg = f"There is no Account with the username '{username}'"
                raise FetchError(msg) from err
            if account.is_active is False:
                msg = (
                    "The Account with the username '{username}' is marked as inactive."
                )
                raise FetchError(msg)
            self.accounts = [account]

    def fetch(self, **kwargs):
        for account in self.accounts:
            self.return_value.append(ScrobblesFetcher(account).fetch(**kwargs))

        return self.return_value



# Generated by Django 1.9.5 on 2016-06-03 12:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0037_user_avatar"),
    ]

    operations = [
        migrations.AddField(
            model_name="tweet",
            name="retweeted_status_id",
            field=models.BigIntegerField(
                blank=True, help_text="The ID of the retweeted Tweet, if any", null=True
            ),
        ),
    ]



from imagekit import ImageSpec, register
from imagekit.processors import Adjust, ResizeToFill, ResizeToFit, Transpose


class TwitterSpec(ImageSpec):
    "Base class for Medium and Small specs."

    format = "JPEG"
    options = {"quality": 80}
    # If original image is smaller, don't enlarge it:
    upscale = False

    def __init__(self, source):
        self.processors = [
            Transpose(),
            ResizeToFit(self.width, self.height, upscale=self.upscale),
            Adjust(sharpness=2.0),
        ]
        super().__init__(source)


# The 'Large' size appears to be the original image, so we're not using
# Imagekit to generate these.
# class Large(TwitterSpec):
# pass
#
# register.generator('ditto_twitter:large', Large)


class Medium(TwitterSpec):
    width = 1200
    height = 1200


register.generator("ditto_twitter:medium", Medium)


class Small(TwitterSpec):
    width = 680
    height = 680


register.generator("ditto_twitter:small", Small)


class Thumbnail(ImageSpec):
    width = 150
    height = 150
    upscale = True

    def __init__(self, source):
        self.processors = [
            Transpose(),
            ResizeToFill(self.width, self.height, upscale=True),
            Adjust(sharpness=2.0),
        ]
        super().__init__(source)


register.generator("ditto_twitter:thumbnail", Thumbnail)



# Generated by Django 3.1.3 on 2020-11-19 15:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0023_auto_20180104_1457"),
    ]

    operations = [
        migrations.AddField(
            model_name="photo",
            name="x_large_3k_height",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="x_large_3k_width",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="x_large_4k_height",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="x_large_4k_width",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="x_large_5k_height",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="x_large_5k_width",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="x_large_6k_height",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="x_large_6k_width",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 1.11.4 on 2017-11-13 10:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0053_tweet_post_year"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="name",
            field=models.CharField(help_text="eg, 'Samuel Pepys'", max_length=50),
        ),
    ]



# Generated by Django 4.1 on 2022-11-28 14:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0026_alter_taggedbookmark_content_type_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bookmarktag",
            name="slug",
            field=models.SlugField(
                allow_unicode=True, max_length=100, unique=True, verbose_name="slug"
            ),
        ),
    ]



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0005_auto_20150730_1350"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="account",
            options={"ordering": ["-time_created"]},
        ),
        migrations.RemoveField(
            model_name="account",
            name="screen_name",
        ),
    ]



# Generated by Django 1.10.1 on 2016-11-17 16:26

from django.db import migrations, models


def set_post_year(apps, schema_editor):
    """
    Re-saves every scrobble, so that its auto-generated `post_year` can
    be set.
    """
    Scrobble = apps.get_model("lastfm", "Scrobble")
    for row in Scrobble.objects.all():
        row.save()


class Migration(migrations.Migration):

    dependencies = [
        ("lastfm", "0005_auto_20161117_1622"),
    ]

    operations = [
        migrations.AddField(
            model_name="scrobble",
            name="post_year",
            field=models.PositiveSmallIntegerField(
                blank=True,
                db_index=True,
                help_text="Set automatically on save",
                null=True,
            ),
        ),
        migrations.RunPython(set_post_year, reverse_code=migrations.RunPython.noop),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0008_remove_user_twitter_id_str"),
    ]

    operations = [
        migrations.AddField(
            model_name="account",
            name="last_fetch_id",
            field=models.BigIntegerField(
                help_text=b"The Twitter ID of the most recent Tweet fetched.",
                null=True,
                blank=True,
            ),
        ),
    ]



import json
import urllib
from datetime import datetime, timezone

import requests

from ditto.core.utils import datetime_now

from .models import Account, Bookmark

PINBOARD_API_ENDPOINT = "https://api.pinboard.in/v1/"
PINBOARD_DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
PINBOARD_ALTERNATE_DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
PINBOARD_DATE_FORMAT = "%Y-%m-%d"


class FetchError(Exception):
    pass


class BookmarksFetcher:
    """The parent class containing common methods.
    Use one of the child classes to fetch a particular set of Bookmarks.
    """

    def fetch(self):
        msg = "Call a child class like AllBookmarksFetcher or RecentBookmarksFetcher"
        raise FetchError(msg)

    def _fetch(self, fetch_type, params=None, username=None):
        """The main method for making all types of Bookmark requests, and
        saving the data.

        Keyword arguments:
        fetch_type -- 'all', 'date', 'recent' or 'url'.
        params -- Any params specific to the type (eg, url='http://foo.com')
                    These will be used directly with the Pinboard API.
        username -- the username of the one Account to fetch (or None for all).
        """
        params = {} if params is None else params
        # Each element will be a dict, like:
        # {'account':'philgyford', 'success':True, 'fetched':12}
        result = []

        accounts = self._get_accounts(username)

        for account in accounts:
            fetch_time = datetime_now()

            response = self._send_request(fetch_type, params, account)

            if response["success"]:
                # Tidy the raw data:
                bookmarks_data = self._parse_response(fetch_type, response["json"])
                # Create/update in DB:
                self._save_bookmarks(
                    account=account,
                    bookmarks_data=bookmarks_data,
                    fetch_time=fetch_time,
                )
                response["fetched"] = len(bookmarks_data)
                # Don't need to pass this around any more:
                del response["json"]
            else:
                response["fetched"] = 0

            response["account"] = account.username
            result.append(response)

        return result

    def _get_accounts(self, username):
        """Get all or one active accounts.
        username is None or a username.
        Raises FetchError if we find that none of the required accounts are
        marked as is_active.
        """
        if username is None:
            accounts = Account.objects.filter(is_active=True)
            if len(accounts) == 0:
                msg = "No active accounts were found to fetch."
                raise FetchError(msg)
        else:
            account = Account.objects.get(username=username)
            if account.is_active:
                accounts = [account]
            else:
                msg = f"The account {username} is curently marked as inactive."
                raise FetchError(msg)
        return accounts

    def _send_request(self, fetch_type, params, account):
        """Sends a request to the Pinboard API, returns the raw response.

        Returns a dict with a 'success' element: True or False.
        If False, will have a 'messages' element, a list.
        If True, will have a 'json' element with the fetched data in.

        Keyword arguments:
        fetch_type -- 'all', 'date', 'recent' or 'url'.
        params -- Any params needed for this type. eg 'dt':datetime.
        Account -- The account to fetch from.
        """

        url_parts = ["posts"]
        if fetch_type in ["date", "url"]:
            url_parts.append("get")
        elif fetch_type == "recent":
            url_parts.append("recent")
        elif fetch_type == "all":
            url_parts.append("all")

        url = "{}{}".format(PINBOARD_API_ENDPOINT, "/".join(url_parts))

        params["format"] = "json"
        params["auth_token"] = account.api_token
        query_string = urllib.parse.urlencode(params)

        final_url = f"{url}?{query_string}"

        error_message = ""

        try:
            response = requests.get(final_url)
        except requests.exceptions.ConnectionError:
            error_message = "Can't connect to domain."
        except requests.exceptions.Timeout:
            error_message = "Connection timed out."
        except requests.exceptions.TooManyRedirects:
            error_message = "Too many redirects."
        except requests.exceptions.RequestException:
            error_message = "Something went wrong with the request."
        else:
            # 2020-06-20. Pinboard changed something on the server that introduced a BOM
            # which caused json.loads() to throw an error. This seems to be the way to
            # ensure this'll work fine if that happens.
            response.encoding = "utf-8-sig"

        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError:
            # 4xx or 5xx errors:
            error_message = f"HTTP Error: {response.status_code}"
        except NameError:
            if error_message == "":
                error_message = "Something unusual went wrong."

        if error_message:
            return {
                "account": account.username,
                "success": False,
                "messages": [error_message],
            }
        else:
            return {"account": account.username, "success": True, "json": response.text}

    def _parse_response(self, fetch_type, json_text):
        """Takes the JSON response for a Bookmark from the API and turns it
        into something more pythony.
        Returns a list of bookmark data.

        Keyword arguments:
        fetch_type -- 'all', 'date', 'recent' or 'url'.
        json_text -- The raw JSON returned from the API.
        """

        response = json.loads(json_text)

        # The JSON has different formats depending on what we fetched:
        posts = response if fetch_type == "all" else response["posts"]

        for bookmark in posts:
            # Before we do anything to it, we give the bookmark a 'json'
            # element with its original state in:
            bookmark["json"] = json.dumps(bookmark)
            # Time string to object:
            bookmark["time"] = datetime.strptime(
                bookmark["time"], "%Y-%m-%dT%H:%M:%SZ"
            ).replace(tzinfo=timezone.utc)
            # 'yes'/'no' to booleans:
            bookmark["shared"] = bookmark["shared"] == "yes"
            bookmark["toread"] = bookmark["toread"] == "yes"
            # String into an array of strings:
            bookmark["tags"] = bookmark["tags"].split()

        return posts

    def _save_bookmarks(self, account, bookmarks_data, fetch_time):
        """Takes the raw JSON response from the API, creates or updates the
        Bookmark objects.

        Keyword arguments:
        account -- The Account object to add these bookmarks for.
        bookmarks_data -- A list, each one data to create a single Bookmark.
        fetch_time -- The UTC time at which these bookmarks were fetched.
        """
        for bookmark in bookmarks_data:
            self._save_bookmark(bookmark, fetch_time, account)

    def _save_bookmark(self, bookmark, fetch_time, account):
        """Takes data for a single bookmark from the API response and creates
        or updates the Bookmark object.

        Keyword arguments:
        bookmark_data -- A list, each one data to create a single Bookmark.
        fetch_time -- The UTC time at which these bookmarks were fetched.
        account -- The Account object to add these bookmarks for.
        """

        bookmark_obj, created = Bookmark.objects.update_or_create(
            account=account,
            url=bookmark["href"],
            defaults={
                "title": bookmark["description"],
                "is_private": not bookmark["shared"],
                "raw": bookmark["json"],
                "description": bookmark["extended"],
                "to_read": bookmark["toread"],
                "fetch_time": fetch_time,
                "post_time": bookmark["time"],
            },
        )

        bookmark_obj.tags.set(bookmark["tags"])


class AllBookmarksFetcher(BookmarksFetcher):
    def fetch(self, username=None):
        """Fetches all of the Bookmarks for all or one Accounts.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        username -- the username of the one Account to fetch (or None for all).
        """
        return self._fetch(fetch_type="all", username=username)


class DateBookmarksFetcher(BookmarksFetcher):
    def fetch(self, post_date, username=None):
        """Fetches Bookmarks for all or one Accounts on a particular day.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        post_date -- date to fetch in a "YYYY-MM-DD" format string.
        username -- the username of the one Account to fetch (or None for all).

        Raises:
        FetchError if the date format is invalid.
        """
        try:
            dt = datetime.strptime(post_date, "%Y-%m-%d").astimezone(timezone.utc)
        except ValueError as err:
            msg = f"Invalid date format ('{post_date}')"
            raise FetchError(msg) from err
        else:
            return self._fetch(fetch_type="date", params={"dt": dt}, username=username)


class RecentBookmarksFetcher(BookmarksFetcher):
    def fetch(self, num=10, username=None):
        """Fetches the most recent Bookmarks for all or one Accounts.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        num -- the number of most recent Bookmarks to fetch.
        username -- the username of the one Account to fetch (or None for all).
        """
        return self._fetch(
            fetch_type="recent", params={"count": int(num)}, username=username
        )


class UrlBookmarksFetcher(BookmarksFetcher):
    def fetch(self, url, username=None):
        """Fetches a single Bookmark (by URL) for all or one Accounts.
        Creates/updates the Bookmark objects.

        Keyword arguments:
        url -- the URL of the Bookmark to fetch.
        username -- the username of the one Account to fetch (or None for all).
        """
        return self._fetch(fetch_type="url", params={"url": url}, username=username)



from django.urls import path, re_path

from . import views

app_name = "twitter"

urlpatterns = [
    path("", view=views.HomeView.as_view(), name="home"),
    path("likes/", view=views.FavoriteListView.as_view(), name="favorite_list"),
    re_path(
        r"^(?P<screen_name>\w+)/$",
        view=views.UserDetailView.as_view(),
        name="user_detail",
    ),
    re_path(
        r"^(?P<screen_name>\w+)/likes/$",
        view=views.AccountFavoriteListView.as_view(),
        name="account_favorite_list",
    ),
    re_path(
        r"^(?P<screen_name>\w+)/(?P<twitter_id>[0-9]+)/$",
        view=views.TweetDetailView.as_view(),
        name="tweet_detail",
    ),
]
